@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: browse.c,v 4.83 1999/10/01 23:20:59 mikes Exp $";
+static char rcsid[] = "$Id: browse.c,v 4.84 1999/12/22 17:56:55 hubert Exp $";
 #endif
 /*
  * Program:	Routines to support file browser in pico and Pine composer
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: browse.c,v 4.83 1999/10/01 23:20:59 mikes Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1998 by the University of Washington.
+ * 1989-1999 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -973,6 +973,11 @@ int   dirlen, fnlen, flags;
 
 	    i = 0;
 	    child[0] = '\0';
+	    /* pass in default filename */
+	    if(fn && *fn){
+		strncpy(child, fn, sizeof(child) - 1);
+		child[sizeof(child) - 1] = '\0';
+	    }
 
 	    while(!i){
 
@@ -991,9 +996,13 @@ int   dirlen, fnlen, flags;
 		    i++;
 		    break;
 		  case FALSE:
-		    i++;
-		    mlerase();
-		    break;
+		    /*
+		     * Support default filename. A return of 'FALSE' means
+		     * 'No change in filename' which is fine if we have
+		     * provided a default.
+		     * John Berthels <john.berthels@nexor.co.uk>
+		     */
+		    /* FALLTHROUGH */
 		  case TRUE:
 		    i++;
 
