@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: display.c,v 4.97 1999/09/01 19:24:33 hubert Exp $";
+static char rcsid[] = "$Id: display.c,v 4.100 2000/11/09 21:53:43 hubert Exp $";
 #endif
 /*
  * Program:	Display functions
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: display.c,v 4.97 1999/09/01 19:24:33 hubert Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1999 by the University of Washington.
+ * 1989-2000 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -39,28 +39,30 @@ static char rcsid[] = "$Id: display.c,v 4.97 1999/09/01 19:24:33 hubert Exp $";
 
 
 #ifdef	ANSI
-    int vtmove(int, int);
-    int vtputc(CELL);
-    int vtpute(CELL);
-    int vteeol(void);
-    int updateline(int, CELL *, CELL *, short *);
-    int updext(void);
-    int mlputi(int, int);
-    int mlputli(long, int);
-    int showCompTitle(void);
+    void vtmove(int, int);
+    void vtputc(CELL);
+    void vtpute(CELL);
+    void vteeol(void);
+    void updateline(int, CELL *, CELL *, short *);
+    void updext(void);
+    void mlputi(int, int);
+    void pprints(int, int);
+    void mlputli(long, int);
+    void showCompTitle(void);
     int nlforw(void);
     int dumbroot(int, int);
     int dumblroot(long, int);
 #else
-    int vtmove();
-    int vtputc();
-    int vtpute();
-    int vteeol();
-    int updateline();
-    int updext();
-    int mlputi();
-    int mlputli();
-    int showCompTitle();
+    void vtmove();
+    void vtputc();
+    void vtpute();
+    void vteeol();
+    void updateline();
+    void updext();
+    void mlputi();
+    void pprints();
+    void mlputli();
+    void showCompTitle();
     int nlforw();
     int dumbroot();
     int dumblroot();
@@ -219,6 +221,7 @@ vtterminalinfo(termcap_wins)
  * system prompt will be written in the line). Shut down the channel to the
  * terminal.
  */
+void
 vttidy()
 {
     movecursor(term.t_nrow-1, 0);
@@ -234,6 +237,7 @@ vttidy()
  * screen. There is no checking for nonsense values; this might be a good
  * idea during the early stages.
  */
+void
 vtmove(row, col)
 int row, col;
 {
@@ -248,6 +252,7 @@ int row, col;
  * only puts printing characters into the virtual terminal buffers. Only
  * column overflow is checked.
  */
+void
 vtputc(c)
 CELL c;
 {
@@ -284,6 +289,7 @@ CELL c;
  * not yet on left edge, don't print it yet. check for overflow on
  * the right margin.
  */
+void
 vtpute(c)
 CELL c;
 {
@@ -323,6 +329,7 @@ CELL c;
  * Erase from the end of the software cursor to the end of the line on which
  * the software cursor is located.
  */
+void
 vteeol()
 {
     register VIDEO      *vp;
@@ -343,6 +350,7 @@ vteeol()
  * correct for the current window. Third, make the virtual and physical
  * screens the same.
  */
+void
 update()
 {
     register LINE   *lp;
@@ -356,7 +364,7 @@ update()
 
 #if	TYPEAH
     if (typahead())
-	return(TRUE);
+	return;
 #endif
 
 #ifdef _WINDOWS
@@ -728,7 +736,7 @@ out:
 #ifdef _WINDOWS
 		mswin_endupdate ();
 #endif
-	        return(TRUE);
+	        return;
 	    }
 #endif
             vp2 = pscreen[i];
@@ -782,6 +790,7 @@ out:
  *	    will be scrolled right or left to let the user see where
  *	    the cursor is
  */
+void
 updext()
 {
     register int rcursor;		/* real cursor location */
@@ -815,6 +824,7 @@ updext()
  * row and column variables. It does try an exploit erase to end of line. The
  * RAINBOW version of this routine uses fast video.
  */
+void
 updateline(row, vline, pline, flags)
 int  row;
 CELL vline[];				/* what we want it to end up as */
@@ -852,7 +862,7 @@ short *flags;				/* and how we want it that way  */
     /* if both lines are the same, no update needs to be done */
     if (cp1 == cp3){
 	*flags &= ~VFCHG;			/* mark it clean */
-	return(TRUE);
+	return;
     }
 
     /* find out if there is a match on the right */
@@ -936,6 +946,7 @@ short *flags;				/* and how we want it that way  */
  * change the modeline format by hacking at this routine. Called by "update"
  * any time there is a dirty window.
  */
+void
 modeline(wp)
 WINDOW *wp;
 {
@@ -1076,6 +1087,7 @@ WINDOW *wp;
  * and column "col". The row and column arguments are origin 0. Optimize out
  * random calls. Update "ttrow" and "ttcol".
  */
+void
 movecursor(row, col)
 int row, col;
 {
@@ -1090,11 +1102,13 @@ int row, col;
 /*
  * Erase any sense we have of the cursor's HW location...
  */
+void
 clearcursor()
 {
     ttrow = ttcol = FARAWAY;
 }
 
+void
 get_cursor(row, col)
     int *row, *col;
 {
@@ -1785,6 +1799,7 @@ void *arg;
  * Write out an integer, in the specified radix. Update the physical cursor
  * position. This will not handle any negative numbers; maybe it should.
  */
+void
 mlputi(i, r)
 int i, r;
 {
@@ -1808,6 +1823,7 @@ int i, r;
 /*
  * do the same except as a long integer.
  */
+void
 mlputli(l, r)
 long l;
 int  r;
@@ -1837,6 +1853,7 @@ int  r;
  *        r  is the row at which to begin scrolling
  *        n  is the number of lines to scrol
  */
+void
 scrolldown(wp, r, n)
 WINDOW *wp;
 int     r, n;
@@ -1879,6 +1896,7 @@ int     r, n;
  *            display, and update the pscreen array to reflect those
  *            moves...
  */
+void
 scrollup(wp, r, n)
 WINDOW *wp;
 int     r, n;
@@ -1922,6 +1940,7 @@ int     r, n;
  * print spaces in the physical screen starting from row abs(n) working in
  * either the positive or negative direction (depending on sign of n).
  */
+void
 pprints(x, y)
 int x, y;
 {
@@ -2091,6 +2110,7 @@ redraw_pico_for_callback()
 /*
  * showCompTitle - display the anchor line passed in from pine
  */
+void
 showCompTitle()
 {
     if(Pmaster){
@@ -2099,7 +2119,7 @@ showCompTitle()
 	COLOR_PAIR *lastc = NULL;
 
 	if((bufp = pico_anchor) == NULL)
-	  return(1);
+	  return;
 	
 	movecursor(COMPOSER_TITLE_LINE, 0);
 	if (Pmaster->colors && Pmaster->colors->tbcp &&
@@ -2131,6 +2151,7 @@ showCompTitle()
 /*
  * zotdisplay - blast malloc'd space created for display maps
  */
+void
 zotdisplay()
 {
     register int i;
@@ -2167,6 +2188,7 @@ nlforw()
  * pputc - output the given char, keep track of it on the physical screen
  *	   array, and keep track of the cursor
  */
+void
 pputc(c, a)
 int   c;				/* char to write */
 int   a;				/* and its attribute */
@@ -2185,6 +2207,7 @@ int   a;				/* and its attribute */
 /*
  * pputs - print a string and keep track of the cursor
  */
+void
 pputs(s, a)
 register char *s;			/* string to write */
 register int   a;			/* and its attribute */
@@ -2198,6 +2221,7 @@ register int   a;			/* and its attribute */
  * peeol - physical screen array erase to end of the line.  remember to
  *	   track the cursor.
  */
+void
 peeol()
 {
     register int r = ttrow;
@@ -2237,6 +2261,7 @@ int l, o;
 /*
  * pclear() - clear the physical screen from row x through row y
  */
+void
 pclear(x, y)
 register int x;
 register int y;
@@ -2414,6 +2439,7 @@ int      key;
  *  NOTE: function key mode is handled here since all the labels
  *        are the same...
  */
+void
 wkeyhelp(keymenu)
 KEYMENU *keymenu;
 {
@@ -2428,7 +2454,7 @@ KEYMENU *keymenu;
 #endif
 
     if(term.t_mrow == 0)
-      return(FALSE);
+      return;
 
     /*
      * Calculate amount of space for the names column by column...
