@@ -1,5 +1,5 @@
 /*
- * $Id: estruct.h,v 4.50 1999/10/04 20:02:31 mikes Exp $
+ * $Id: estruct.h,v 4.51 2000/06/26 21:39:18 hubert Exp $
  *
  * Program:	Struct and preprocessor definitions
  *
@@ -20,7 +20,7 @@
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1999 by the University of Washington.
+ * 1989-2000 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -103,6 +103,15 @@
 #define	FIOPER	10			/* Generic permission denied    */
 
 
+/* for can_access (access) function */
+#define EXECUTE_ACCESS	01		/* These five are 	   */
+#define WRITE_ACCESS	02		/*    for the calls	   */
+#define READ_ACCESS	04		/*       to access()       */
+#define ACCESS_EXISTS	00		/*           <etc>         */
+#define EDIT_ACCESS	06		/*  (this is r+w access)   */
+#define OWNER_ONLY	010		/* open with mode 0600     */
+
+
 #define CFCPCN  0x0001                  /* Last command was C-P, C-N    */
 #define CFKILL  0x0002                  /* Last command was a kill      */
 #define CFFILL  0x0004                  /* Last command was a kill      */
