@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: file.c,v 4.70 1999/03/23 01:25:05 jfrankli Exp $";
+static char rcsid[] = "$Id: file.c,v 4.73 2000/12/28 23:21:38 hubert Exp $";
 #endif
 /*
  * Program:	High level file input and output routines
@@ -77,7 +77,7 @@ int f, n;
 	  return(0);
 	}
 
-        return(readin(fname, TRUE));
+        return(readin(fname, TRUE, TRUE));
 }
 
 
@@ -396,9 +396,10 @@ insmsgchar(c)
  * to read in a file specified on the command line as
  * an argument.
  */
-readin(fname, lockfl)
+readin(fname, lockfl, rename)
 char    fname[];	/* name of file to read */
 int	lockfl;		/* check for file locks? */
+int     rename;         /* don't rename if reading from, say, alt speller */
 {
         char line[NLINE], *linep;
 	long nline;
@@ -408,7 +409,8 @@ int	lockfl;		/* check for file locks? */
 	if ((s = bclear(curbp)) != TRUE)	/* Might be old.        */
 	  return (s);
 
-	strcpy(curbp->b_fname, fname);
+	if(rename)
+	  strcpy(curbp->b_fname, fname);
 	if ((s=ffropen(fname)) != FIOSUC){	/* Hard file open.      */
 	    if(s == FIOFNF)                     /* File not found.      */
 	      emlwrite("New file", NULL);
@@ -585,7 +587,7 @@ int f, n;
 		  strcpy(shows, ((gmode & MDTREE) || opertree[0])
 				   ? opertree : gethomedir(NULL));
 
-		if (s = FileBrowse(shows, NLINE, fname, NFILEN, NULL, FB_SAVE) == 1) {
+		if ((s = FileBrowse(shows, NLINE, fname, NFILEN, NULL, FB_SAVE)) == 1) {
 		  if (strlen(shows)+strlen(S_FILESEP)+strlen(fname) < NLINE){
 		    strcat(shows, S_FILESEP);
 		    strcat(shows, fname);
@@ -767,8 +769,13 @@ char *dir;
 
         tmpname(dir, fn);
 
-        if ((s=ffwopen(fn, TRUE)) != FIOSUC)	/* Open writes message. */
-          return(NULL);
+	/* Open writes message */
+        if (!fn[0] || (s=ffwopen(fn, TRUE)) != FIOSUC){
+	    if(fn[0])
+	      (void)unlink(fn);
+
+	    return(NULL);
+	}
 
         lp = lforw(curbp->b_linep);             /* First line.          */
         nline = 0;                              /* Number of lines.     */
@@ -787,7 +794,7 @@ char *dir;
 	  s = t;				/* remember worst case */
 
 	if (s != FIOSUC){                       /* Some sort of error.  */
-	    unlink(fn);
+	    (void)unlink(fn);
 	    return(NULL);
 	}
 
