@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: line.c,v 4.21 1999/10/20 19:10:44 mikes Exp $";
+static char rcsid[] = "$Id: line.c,v 4.22 2000/11/09 21:53:43 hubert Exp $";
 #endif
 /*
  * Program:	Line management routines
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: line.c,v 4.21 1999/10/20 19:10:44 mikes Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1998 by the University of Washington.
+ * 1989-2000 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -63,16 +63,16 @@ static struct pkbuf {
 
 
 #ifdef	ANSI
-    int insspace(int, int);
+    void insspace(int, int);
     int ldelnewline(void);
-    int pkbufdel(struct pkbuf **);
+    void pkbufdel(struct pkbuf **);
     void pkchunkdel(struct pkchunk **);
     int pkbufinsert(int, struct pkbuf **);
     int pkbufremove(int, struct pkbuf *);
 #else
-    int insspace();
+    void insspace();
     int ldelnewline();
-    int pkbufdel();
+    void pkbufdel();
     void pkchunkdel();
     int pkbufinsert();
     int pkbufremove();
@@ -114,6 +114,7 @@ register int    used;
  * might be in. Release the memory. The buffers are updated too; the magic
  * conditions described in the above comments don't hold here.
  */
+void
 lfree(lp)
 register LINE   *lp;
 {
@@ -168,6 +169,7 @@ register LINE   *lp;
  * displayed in more than 1 window we change EDIT t HARD. Set MODE if the
  * mode line needs to be updated (the "*" has to be set).
  */
+void
 lchange(flag)
 register int    flag;
 {
@@ -195,6 +197,7 @@ register int    flag;
  * insert spaces forward into text
  * default flag and numeric argument 
  */
+void
 insspace(f, n)	
 int f, n;
 {
@@ -630,16 +633,19 @@ lisblank(line)
  * new kill context is being created. The kill buffer array is released, just
  * in case the buffer has grown to immense size. No errors.
  */
+void
 kdelete()
 {
-    return(pkbufdel(&kbufp));
+    pkbufdel(&kbufp);
 }
 
+void
 fdelete()
 {
-    return(pkbufdel(&fbufp));
+    pkbufdel(&fbufp);
 }
 
+void
 pkbufdel(buf)
     struct pkbuf **buf;
 {
