@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: main.c,v 4.63 1999/09/23 21:43:52 mikes Exp $";
+static char rcsid[] = "$Id: main.c,v 4.65 2000/05/19 17:42:43 hubert Exp $";
 #endif
 /*
  * Program:	Main stand-alone Pine Composer routines
@@ -88,6 +88,7 @@ char *args_pico_args[] = {
 "\tArgument\t\tMeaning",
 "\t -e \t\tComplete - allow file name completion",
 "\t -k \t\tCut - let ^K cut from cursor position to end of line",
+"\t -a \t\tShowDot - show dot files in file browser",
 "\t -j \t\tGoto - allow 'Goto' command in file browser",
 "\t -g \t\tShow - show cursor in file browser",
 "\t -m \t\tMouse - turn on mouse support",
@@ -204,7 +205,7 @@ char    *argv[];
 	else{
 	    strcpy(bp->b_fname, file_to_edit);
 	    if ((gmode&MDTREE) && !in_oper_tree(file_to_edit) ||
-		readin(file_to_edit, (viewflag==FALSE)) == ABORT) {
+		readin(file_to_edit, (viewflag==FALSE), TRUE) == ABORT) {
 		if ((gmode&MDTREE) && !in_oper_tree(file_to_edit))
 		  emlwrite("Can't read file from outside of %s", opertree);
 
@@ -417,53 +418,56 @@ Loop:
 	  /*
 	   * These don't take arguments.
 	   */
-	  case 'v':			/* -v for View File */
-	  case 'V':
-	    *viewflag = !*viewflag;
-	    break;			/* break back to inner-while */
-	  case 'e':			/* file name completion */
-	    gmode ^= MDCMPLT;
-	    break;
-	  case 'k':			/* kill from dot */
-	    gmode ^= MDDTKILL;
-	    break;
-	  case 'j':			/* allow "Goto" in file browser */
-	    gmode ^= MDGOTO;
-	    break;
-	  case 'g':			/* show-cursor in file browser */
-	    gmode ^= MDSHOCUR;
-	    break;
-	  case 'm':			/* turn on mouse support */
-	    gmode ^= MDMOUSE;
-	    break;
-	  case 'x':			/* suppress keyhelp */
-	    sup_keyhelp = !sup_keyhelp;
+          case 'a':
+            gmode ^= MDDOTSOK;          /* show dot files */
+            break;
+	  case 'b':
+	    gmode ^= MDREPLACE;  /* -b for replace string in where is command */
 	    break;
 	  case 'd':			/* -d for rebind delete key */
 	    bindtokey(0x7f, forwdel);
 	    break;
+	  case 'e':			/* file name completion */
+	    gmode ^= MDCMPLT;
+	    break;
 	  case 'f':			/* -f for function key use */
 	    gmode ^= MDFKEY;
 	    break;
-	  case 'b':
-	    gmode ^= MDREPLACE;  /* -b for replace string in where is command */
+	  case 'g':			/* show-cursor in file browser */
+	    gmode ^= MDSHOCUR;
 	    break;
 	  case 'h':
 	    usage++;
 	    break;
+	  case 'j':			/* allow "Goto" in file browser */
+	    gmode ^= MDGOTO;
+	    break;
+	  case 'k':			/* kill from dot */
+	    gmode ^= MDDTKILL;
+	    break;
+	  case 'm':			/* turn on mouse support */
+	    gmode ^= MDMOUSE;
+	    break;
+	  case 'q':			/* -q for termcap takes precedence */
+	    gmode ^= MDTCAPWINS;
+	    break;
 	  case 't':			/* special shutdown mode */
 	    gmode ^= MDTOOL;
 	    rebindfunc(wquit, quickexit);
 	    break;
-	  case 'z':			/* -z to suspend */
-	    gmode ^= MDSSPD;
-	    break;
-	  case 'q':			/* -q for termcap takes precedence */
-	    gmode ^= MDTCAPWINS;
-	    break;
+	  case 'v':			/* -v for View File */
+	  case 'V':
+	    *viewflag = !*viewflag;
+	    break;			/* break back to inner-while */
 	  case 'w':			/* -w turn off word wrap */
 	    gmode ^= MDWRAP;
 	    break;
+	  case 'x':			/* suppress keyhelp */
+	    sup_keyhelp = !sup_keyhelp;
+	    break;
+	  case 'z':			/* -z to suspend */
+	    gmode ^= MDSSPD;
+	    break;
 
 	  /*
 	   * These do take arguments.
@@ -567,7 +571,7 @@ pico_file_drop(x, y, filename)
 	strcpy(bp->b_bname, bname);
 	strcpy(bp->b_fname, filename);
 	bp->b_flag &= ~BFCHG;	/* turn off change bit */
-	if (readin(filename, 1) == ABORT) {
+	if (readin(filename, 1, 1) == ABORT) {
 	    strcpy(bp->b_bname, "");
 	    strcpy(bp->b_fname, "");
 	}
