@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: pico.c,v 4.121 1999/06/10 22:56:11 mikes Exp $";
+static char rcsid[] = "$Id: pico.c,v 4.124 2000/11/09 21:53:43 hubert Exp $";
 #endif
 /*
  * Program:	Main Pine Composer routines
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: pico.c,v 4.121 1999/06/10 22:56:11 mikes Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1998 by the University of Washington.
+ * 1989-2000 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -70,9 +70,10 @@ static char rcsid[] = "$Id: pico.c,v 4.121 1999/06/10 22:56:11 mikes Exp $";
 #include	"ebind.h"	/* default key bindings */
 
 
-int	func_init PROTO((void));
+void	func_init PROTO((void));
 void	breplace PROTO((void *));
 int	any_header_changes PROTO((void));
+int     stripwhitespace PROTO((void));
 #ifdef	_WINDOWS
 int	composer_file_drop PROTO((int, int, char *));
 #endif
@@ -215,6 +216,7 @@ PICO *pm;
 	    switch(pico_all_done){	/* prepare for/handle final events */
 	      case COMP_EXIT :		/* already confirmed */
 		packheader();
+		stripwhitespace();
 		c |= COMP_EXIT;
 		break;
 
@@ -399,6 +401,7 @@ PICO *pm;
  * for editing.  the first some odd lines are to be used for message 
  * header information editing.
  */
+void
 edinit(bname)
 char    bname[];
 {
@@ -413,7 +416,7 @@ char    bname[];
 
     if (bp==NULL || wp==NULL){
 	if(Pmaster)
-	  return(0);
+	  return;
 	else
 	  exit(1);
     }
@@ -569,6 +572,7 @@ int c, f, n;
  * Fancy quit command, as implemented by Norm. If the any buffer has
  * changed do a write on that buffer and exit emacs, otherwise simply exit.
  */
+int
 quickexit(f, n)
 int f, n;
 {
@@ -583,7 +587,7 @@ int f, n;
 	}
 	bp = bp->b_bufp;			/* on to the next buffer */
     }
-    wquit(f, n);                             /* conditionally quit   */
+    return(wquit(f, n));                             /* conditionally quit   */
 }
 
 
@@ -649,6 +653,8 @@ int f, n;
       pico_all_done = COMP_SUSPEND;
     else
       (*term.t_beep)();
+    
+    return(TRUE);
 }
 
 
@@ -739,6 +745,29 @@ any_header_changes()
     return(he->name && he->dirty);
 }
 
+/*
+ * Remove all trailing white space from the text
+ */
+int
+stripwhitespace()
+{
+    int i;
+    LINE *cur_line = lforw(curbp->b_linep);
+
+    do{
+      /* we gotta test for the sigdash case here */
+      if(!(cur_line->l_used == 3 && 
+	   lgetc(cur_line, 0).c == '-' &&
+	   lgetc(cur_line, 1).c == '-' &&
+	   lgetc(cur_line, 2).c == ' '))
+	for(i = cur_line->l_used - 1; i >= 0; i--)
+	  if(isspace(lgetc(cur_line, i).c))
+	    cur_line->l_used--;
+	  else
+	    break;
+    }while((cur_line = lforw(cur_line)) != curbp->b_linep);
+    return 0;
+}
 
 /*
  * Abort.
@@ -768,6 +797,7 @@ rdonly()
 /*
  * reset all globals to their initial values
  */
+void
 func_init()
 {
     extern int    vtrow;
@@ -816,6 +846,7 @@ int i;
 /*
  * zotedit() - kills the buffer and frees all lines associated with it!!!
  */
+void
 zotedit()
 {
     wheadp->w_linep = wheadp->w_dotp = wheadp->w_markp = NULL;
@@ -1446,8 +1477,12 @@ pico_puts(w, s)
 void *w;
 char *s;
 {
+    int rv = 0;
+
     while(*s != '\0')
-      pico_writec(w, (int)*s++);
+      rv += pico_writec(w, (int)*s++);
+
+    return((rv) ? 1 : 0);
 }
 
 
