@@ -1,5 +1,5 @@
 /*
- * $Id: pico.h,v 4.111 1999/10/20 19:11:56 mikes Exp $
+ * $Id: pico.h,v 4.116 2001/01/02 22:14:27 jpf Exp $
  *
  * Program:	pico.h - definitions for Pine's composer library
  *
@@ -164,6 +164,7 @@ typedef struct pico_colors {
     COLOR_PAIR *tbcp;                   /* title bar color pair */
     COLOR_PAIR *klcp;                   /* key label color pair */
     COLOR_PAIR *kncp;                   /* key name color pair  */
+    COLOR_PAIR *stcp;                   /* status color pair    */
 } PCOLORS;
 
 /*
@@ -403,7 +404,7 @@ extern	void	kbdestroy PROTO((KBESC_T *));
  */
 #ifdef	maindef
 PICO	*Pmaster = NULL;		/* composer specific stuff */
-char	*version = "3.7";		/* PICO version number */
+char	*version = "4.0";		/* PICO version number */
 
 #else
 extern	PICO *Pmaster;			/* composer specific stuff */
@@ -415,8 +416,9 @@ extern	char *version;			/* pico version! */
 /*
  * Flags for FileBrowser call
  */
-#define FB_READ		0x0001		/* Looking for a file to read. */
-#define FB_SAVE		0x0002		/* Looking for a file to save. */
+#define FB_READ		0x0001		/* Looking for a file to read.  */
+#define FB_SAVE		0x0002		/* Looking for a file to save.  */
+#define	FB_ATTACH	0x0004		/* Looking for a file to attach */
 
 
 /*
