@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: random.c,v 4.18 1998/04/02 06:00:51 mikes Exp $";
+static char rcsid[] = "$Id: random.c,v 4.20 1999/06/23 18:33:02 hubert Exp $";
 #endif
 /*
  * Program:	Random routines
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: random.c,v 4.18 1998/04/02 06:00:51 mikes Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1998 by the University of Washington.
+ * 1989-1999 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -377,3 +377,51 @@ int f, n;
 
     return (TRUE);
 }
+
+
+/*
+ * Returns a pointer to the name of color number color.
+ * The colors 0 through 7 are the names of the ANSI colors, then
+ * come the strings color008, color009, color010, ...
+ *
+ * Warning, returns a pointer to a static buffer, so the result must be
+ * copied before another call to colorx.
+ */
+char *
+colorx(color)
+    int color;
+{
+    static char cbuf[12];
+
+    switch(color){
+      case COL_BLACK:
+	strcpy(cbuf, "black");
+	break;
+      case COL_RED:
+	strcpy(cbuf, "red");
+	break;
+      case COL_GREEN:
+	strcpy(cbuf, "green");
+	break;
+      case COL_YELLOW:
+	strcpy(cbuf, "yellow");
+	break;
+      case COL_BLUE:
+	strcpy(cbuf, "blue");
+	break;
+      case COL_MAGENTA:
+	strcpy(cbuf, "magenta");
+	break;
+      case COL_CYAN:
+	strcpy(cbuf, "cyan");
+	break;
+      case COL_WHITE:
+	strcpy(cbuf, "white");
+	break;
+      default:
+        sprintf(cbuf, "color%03.3d", color);
+	break;
+    }
+
+    return(cbuf);
+}
