@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: region.c,v 4.19 1996/07/09 23:55:54 mikes Exp $";
+static char rcsid[] = "$Id: region.c,v 4.23 1998/04/02 06:01:27 mikes Exp $";
 #endif
 /*
  * Program:	Region management routines
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: region.c,v 4.19 1996/07/09 23:55:54 mikes Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1996 by the University of Washington.
+ * 1989-1998 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -34,11 +34,7 @@ static char rcsid[] = "$Id: region.c,v 4.19 1996/07/09 23:55:54 mikes Exp $";
  * commands. Some functions are just for
  * internal use.
  */
-#include        <stdio.h>
-#include	"osdep.h"
-#include	"pico.h"
-#include	"estruct.h"
-#include        "edef.h"
+#include	"headers.h"
 
 /*
  * Kill the region. Ask "getregion"
@@ -47,23 +43,14 @@ static char rcsid[] = "$Id: region.c,v 4.19 1996/07/09 23:55:54 mikes Exp $";
  * Bound to "C-W".
  */
 killregion(f, n)
+    int f, n;
 {
     REGION          region;
-    static long     times = 1L;
-    static long     backoff = 4L;
 
     if (curbp->b_mode&MDVIEW)		/* don't allow this command if	*/
       return(rdonly());			/* we are in read only mode	*/
 
-    if (getregion(&region) != TRUE){
-	if((lastflag&CFKILL) == 0){
-	    emlwrite("Line Deleted.%s", !(times % backoff) ? 
-		     " (Could also use ctrl-^ to mark text for cutting)" : "");
-
-	    if(!(times++ % backoff)) 	/* if message shown, reset backoff */
-	      backoff = backoff << 1;
-	}
-	
+    if (getregion(&region, curwp->w_markp, curwp->w_marko) != TRUE){
 	return (killtext(f, n));
     }else {
 	mlerase();
@@ -80,7 +67,7 @@ killregion(f, n)
     mswin_allowcopycut(NULL);
 #endif
 
-    if(ldelete(region.r_size, TRUE)){
+    if(ldelete(region.r_size, kinsert)){
 	if(curwp->w_dotp == curwp->w_linep && curwp->w_dotp == curbp->b_linep){
 	    curwp->w_force = 0;		/* Center dot. */
 	    curwp->w_flag |= WFFORCE;
@@ -100,20 +87,21 @@ killregion(f, n)
  * Bound to "C-W".
  */
 deleteregion(f, n)
+    int f, n;
 {
     REGION          region;
 
     if (curbp->b_mode&MDVIEW)		/* don't allow this command if	*/
       return(rdonly());			/* we are in read only mode	*/
 
-    if (getregion(&region) == TRUE){
+    if (getregion(&region, curwp->w_markp, curwp->w_marko) == TRUE){
 	curwp->w_dotp = region.r_linep;
 	curwp->w_doto = region.r_offset;
 	curwp->w_markp = NULL;
 #ifdef	_WINDOWS
 	mswin_allowcopycut(NULL);
 #endif
-	if(ldelete(region.r_size, FALSE)){
+	if(ldelete(region.r_size, NULL)){
 	    if(curwp->w_dotp == curwp->w_linep
 	       && curwp->w_dotp == curbp->b_linep){
 		curwp->w_force = 0;		/* Center dot. */
@@ -141,7 +129,7 @@ copyregion(f, n)
     register int    s;
     REGION          region;
 
-    if ((s=getregion(&region)) != TRUE)
+    if ((s=getregion(&region, curwp->w_markp, curwp->w_marko)) != TRUE)
       return (s);
 
     if ((lastflag&CFKILL) == 0)		/* Kill type command.   */
@@ -188,7 +176,7 @@ lowerregion(f, n)
     if (curbp->b_mode&MDVIEW)	/* don't allow this command if	*/
       return(rdonly());	/* we are in read only mode	*/
 
-    if ((s=getregion(&region)) != TRUE)
+    if ((s=getregion(&region, curwp->w_markp, curwp->w_marko)) != TRUE)
       return (s);
 
     lchange(WFHARD);
@@ -232,7 +220,7 @@ upperregion(f, n)
     if (curbp->b_mode&MDVIEW)	/* don't allow this command if	*/
       return(rdonly());	/* we are in read only mode	*/
 
-    if ((s=getregion(&region)) != TRUE)
+    if ((s=getregion(&region, curwp->w_markp, curwp->w_marko)) != TRUE)
       return (s);
 
     lchange(WFHARD);
@@ -266,26 +254,28 @@ upperregion(f, n)
  * an "ABORT" status; we might make this have the
  * conform thing later.
  */
-getregion(rp)
+getregion(rp, markp, marko)
 register REGION *rp;
+register LINE	*markp;
+register int	 marko;
 {
     register LINE   *flp;
     register LINE   *blp;
     long	    fsize;
     register long   bsize;
 
-    if (curwp->w_markp == NULL) {
+    if (markp == NULL) {
 	return (FALSE);
     }
 
-    if (curwp->w_dotp == curwp->w_markp) {
+    if (curwp->w_dotp == markp) {
 	rp->r_linep = curwp->w_dotp;
-	if (curwp->w_doto < curwp->w_marko) {
+	if (curwp->w_doto < marko) {
 	    rp->r_offset = curwp->w_doto;
-	    rp->r_size = curwp->w_marko-curwp->w_doto;
+	    rp->r_size = marko - curwp->w_doto;
 	} else {
-	    rp->r_offset = curwp->w_marko;
-	    rp->r_size = curwp->w_doto-curwp->w_marko;
+	    rp->r_offset = marko;
+	    rp->r_size = curwp->w_doto - marko;
 	}
 	return (TRUE);
     }
@@ -297,10 +287,10 @@ register REGION *rp;
     while (flp!=curbp->b_linep || lback(blp)!=curbp->b_linep) {
 	if (flp != curbp->b_linep) {
 	    flp = lforw(flp);
-	    if (flp == curwp->w_markp) {
+	    if (flp == markp) {
 		rp->r_linep = curwp->w_dotp;
 		rp->r_offset = curwp->w_doto;
-		rp->r_size = fsize + curwp->w_marko;
+		rp->r_size = fsize + marko;
 		return (TRUE);
 	    }
 
@@ -310,10 +300,10 @@ register REGION *rp;
 	if (lback(blp) != curbp->b_linep) {
 	    blp = lback(blp);
 	    bsize += llength(blp)+1;
-	    if (blp == curwp->w_markp) {
+	    if (blp == markp) {
 		rp->r_linep = blp;
-		rp->r_offset = curwp->w_marko;
-		rp->r_size = bsize - curwp->w_marko;
+		rp->r_offset = marko;
+		rp->r_size = bsize - marko;
 		return (TRUE);
 	    }
 	}
@@ -336,7 +326,7 @@ markregion(attr)
     REGION          region;
     CELL            ac;
 
-    if ((s=getregion(&region)) != TRUE)
+    if ((s=getregion(&region, curwp->w_markp, curwp->w_marko)) != TRUE)
       return (s);
 
     lchange(WFHARD);
