@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: search.c,v 4.24 1999/10/04 21:49:15 mikes Exp $";
+static char rcsid[] = "$Id: search.c,v 4.25 2000/11/09 21:53:43 hubert Exp $";
 #endif
 /*
  * Program:	Searching routines
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: search.c,v 4.24 1999/10/04 21:49:15 mikes Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1998 by the University of Washington.
+ * 1989-2000 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -40,6 +40,7 @@ static char rcsid[] = "$Id: search.c,v 4.24 1999/10/04 21:49:15 mikes Exp $";
 int	eq PROTO((int, int));
 int	expandp PROTO((char *, char *, int));
 int	readnumpat PROTO((char *));
+void	get_pat_cases PROTO((char *, char *));
 
 
 #define	FWS_RETURN(RV)	{				\
@@ -419,6 +420,7 @@ int  *wrapt;
 /* Since the search is not case sensitive, we must obtain the actual pattern 
    that appears in the text, so that we can highlight (and unhighlight) it
    without using the wrong cases  */
+void
 get_pat_cases(realpat, searchpat)
 char *searchpat, *realpat;
 {
