@@ -1,5 +1,5 @@
 /*----------------------------------------------------------------------
-  $Id: adrbklib.h,v 4.132 1999/09/09 21:41:42 hubert Exp $
+  $Id: adrbklib.h,v 4.137 2001/01/30 23:23:08 hubert Exp $
 
             T H E    P I N E    M A I L   S Y S T E M
 
@@ -20,7 +20,7 @@
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-1999 by the University of Washington.
+   1989-2000 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -253,7 +253,6 @@ typedef unsigned a_c_arg_t;          /* type of arg passed for addrbk_cntr_t */
 
 typedef long adrbk_uid_t;   /* the UID of a name or address */
 
-typedef enum {ReadOnly, ReadWrite, MaybeRorW, NoAccess, NoExists} AccessType;
 typedef enum {NotSet, Single, List} Tag;
 typedef enum {Normal, Delete, SaveDelete, Lock, Unlock} Handling;
 
@@ -320,33 +319,8 @@ typedef struct expanded_list {
     struct expanded_list *next;
 } EXPANDED_S;
 
-/* for RemoteViaImap access */
-typedef struct imap_stuff {
-    MAILSTREAM	 *stream;	/* stream to use for accessing remote folder */
-    time_t        last_use;	/* when stream was last used                 */
-    time_t        last_valid_chk;/* when imap valid check was done           */
-    STORE_S	 *so;		/* SO to use for accessing local cache copy  */
-    char	 *chk_date;	/* Date in message, to help see if it changed*/
-    unsigned long chk_nmsgs;	/* Number of messages in remote folder       */
-    unsigned long uidvalidity;	/* UIDVALIDITY of folder                     */
-    unsigned long uidnext;	/* UIDNEXT of folder                         */
-    unsigned long uid;		/* UID of last message in folder             */
-    char          read_status;	/* R for readonly                            */
-}IMAP_S;
-
 typedef enum {Local, Imap} AdrbkType;
 
-/* For AdrBk member "flags" */
-#define DEL_FILE	0x1	/* remove addrbook file in adrbk_close     */
-#define DEL_HASHFILE	0x2	/* remove abook hashfile in adrbk_close    */
-#define DO_IMAPTRIM	0x4	/* only if it is R/W, Imap, and contacted  */
-#define USE_OLD_CACHE	0x8	/* using cache copy, couldn't check if old */
-#define FILE_OUTOFDATE	0x10	/* addrbook file discovered out of date    */
-#define IMAP_OUTOFDATE	0x20	/* remote imap folder changed since copied */
-#define NO_STATUSCMD	0x40	/* imap server doesn't have status command */
-#define NO_META_UPDATE	0x80	/* don't try to update metafile            */
-#define OUT_OF_SORTS	0x100	/* sort order is messed up                 */
-#define FIRST_CHANGE	0x200	/* first change this session               */
 
 #define IMAP_IDLE_TIMEOUT	(10L * 60L)	/* seconds */
 #define FILE_VALID_CHK_INTERVAL (      15L)	/* seconds */
@@ -357,10 +331,8 @@ typedef struct adrbk {
     char         *orig_filename;       /* passed in filename                 */
     char         *filename;            /* addrbook filename                  */
     char         *our_filecopy;        /* session copy of filename contents  */
-    char         *temp_filename;       /* tmp file while writing out changes */
     char         *hashfile;            /* lookup index file (.lu file)       */
     char         *our_hashcopy;        /* session copy of hashfile contents  */
-    char         *temp_hashfile;       /* tmp file while writing out changes */
     FILE         *fp;                  /* fp for our_filecopy                */
     FILE         *fp_hash;             /* fp for our_hashcopy                */
     AdrHash      *hash_by_nick;
@@ -378,7 +350,7 @@ typedef struct adrbk {
     int          *n_ae_cached_in_this_bucket; /* and n cached in each bucket */
     long          nominal_max_cached;
     adrbk_cntr_t  er_hashsize;
-    unsigned      flags;               /* see defines above                  */
+    unsigned      flags;               /* see defines in pine.h (DEL_FILE...)*/
     WIDTH_INFO_S  widths;              /* helps addrbook.c format columns    */
     int           sort_rule;
     EXPANDED_S   *exp;                 /* this is for addrbook.c to use.  A
@@ -387,19 +359,9 @@ typedef struct adrbk {
 			       list of checked entry nums is kept here */
     EXPANDED_S   *selects;             /* this is for addrbook.c to use.  A
 			       list of selected entry nums is kept here */
-    IMAP_S        imap;
+    REMDATA_S    *rd;
 } AdrBk;
 
-typedef struct remote_ab {
-    char         *local_cache_file;
-    unsigned long uidvalidity;
-    unsigned long uidnext;
-    unsigned long uid;
-    unsigned long nmsgs;
-    char          read_status;	/* 'R' for readonly, 'W' for readwrite */
-    char         *date;
-} REMOTE_AB_S;
-
 
 /*
  * The definitions starting here have to do with the virtual scrolling
@@ -801,7 +763,8 @@ typedef struct addrscreenstate {
                    zoomed,       /* zoomed into view only selected entries */
                    config,       /* called from config screen             */
                    n_serv,       /* how many directory servers are there  */
-                   n_impl;       /* how many of those have impl bit set   */
+                   n_impl,       /* how many of those have impl bit set   */
+                   selected_is_history;
 #ifdef	_WINDOWS
     long	   last_ent;	 /* index of last known entry		  */
 #endif
@@ -817,8 +780,8 @@ typedef struct addrscreenstate {
  * SelectNicksCom return a comma-separated list of nicknames.
  * SelectNickTake, SelectNickCom, and SelectNick all return a single
  * nickname.  The ones that returns multiple nicknames or multiple
- * addresses all allow ListMode. They are SelectAddrLccCom, and
- * SelectNicksCom.
+ * addresses all allow ListMode. They are SelectAddrLccCom,
+ * SelectNicksCom, and SelectMultNoFull.
  */
 typedef enum {AddrBookScreen,	   /* maintenance screen                     */
 	      AddrBookConfig,	   /* config screen                          */
@@ -830,7 +793,8 @@ typedef enum {AddrBookScreen,	   /* maintenance screen                     */
 	      SelectNickCom,	   /* Same as SelectNick but from composer   */
 	      SelectManyNicks,	   /* Returns list of nicks */
 	      SelectAddr,	   /* Returns single address */
-	      SelectAddrNoFull	   /* Returns single address without fullname */
+	      SelectAddrNoFull,	   /* Returns single address without fullname */
+	      SelectMultNoFull	   /* Returns mult addresses without fullname */
 	     } AddrBookArg;
 
 typedef struct save_state_struct {
@@ -911,9 +875,6 @@ extern int          ab_nesting_level;
 #define MAX_CHARS_IN_HASH 75
 #define DELETED "#DELETED-"
 #define DELETED_LEN 9
-#define REMOTE_ABOOK_TYPE	TYPETEXT
-#define REMOTE_ABOOK_SUBTYPE	"x-pine-addrbook"
-#define REMOTE_ABOOK_VERS_NUM	1
 
 
 #define ONE_HUNDRED_DAYS (60L * 60L * 24L * 100L)
@@ -1038,20 +999,16 @@ typedef struct _cust_filt {
  * 47 bytes per array element (counting the newline).
  */
 
+/* Some of the definitions (e.g., PMAGIC) are in pine.h. */
 #define ADRHASH_FILE_SUFFIX      ".lu"
 #ifdef HUGE_ADDRBOOKS
 #define ADRHASH_FILE_VERSION_NUM "14"
 #else
 #define ADRHASH_FILE_VERSION_NUM "13"
 #endif
-#define PMAGIC                   "P#*E@"
 #define LEGACY_PMAGIC            "P#*@ "  /* sorry about that */
-#define METAFILE_VERSION_NUM     "01"
 
-#define SIZEOF_PMAGIC       (5)
-#define SIZEOF_SPACE        (1)
 #define SIZEOF_NEWLINE      (1)
-#define SIZEOF_VERSION_NUM  (2)
 #define SIZEOF_SORT_RULE    (2)
 #define SIZEOF_WIDTH        (2)
 #define SIZEOF_ASCII_USHORT (5)
@@ -1076,8 +1033,6 @@ typedef struct _cust_filt {
 
 #define SIZEOF_COUNT        SIZEOF_HASH_INDEX
 
-#define TO_FIND_HDR_PMAGIC  (0)
-#define TO_FIND_VERSION_NUM (TO_FIND_HDR_PMAGIC + SIZEOF_PMAGIC + SIZEOF_SPACE)
 #define TO_FIND_HTABLE_SIZE (TO_FIND_VERSION_NUM + \
 			     SIZEOF_VERSION_NUM + SIZEOF_SPACE)
 #define SIZEOF_HDR (TO_FIND_HTABLE_SIZE + \
@@ -1124,14 +1079,10 @@ int            adrbk_check_and_fix_all PROTO((int, int, int));
 void           adrbk_check_validity PROTO((AdrBk *, long));
 int            adrbk_check_all_validity_now PROTO((void));
 char          *addr_book_nick_for_edit PROTO((char **));
-void           adrbk_update_metafile PROTO((char *, char *, unsigned long,
-					    unsigned long, unsigned long,
-					    unsigned long, int, char *, int));
 int            adrbk_write PROTO((AdrBk *, adrbk_cntr_t *, int, int, int));
 ADDRESS       *abe_to_address PROTO((AdrBk_Entry *, AddrScrn_Disp *,
                                      AdrBk *, int *));
 int            abes_are_equal PROTO((AdrBk_Entry *, AdrBk_Entry *));
-int            add_header_message PROTO((STORE_S *, char *, char *));
 char          *addr_book_selnick PROTO((void));
 char          *addr_book_takeaddr PROTO((void));
 char          *addr_lookup PROTO((char *, int *, int));
