@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: args.c,v 4.84 1999/11/08 23:00:10 mikes Exp $";
+static char rcsid[] = "$Id: args.c,v 4.103 2000/11/09 22:26:47 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: args.c,v 4.84 1999/11/08 23:00:10 mikes Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-1999 by the University of Washington.
+   1989-2000 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -56,8 +56,6 @@ static char rcsid[] = "$Id: args.c,v 4.84 1999/11/08 23:00:10 mikes Exp $";
 
 int  process_debug_str PROTO((char *));
 void args_add_attach PROTO((PATMT **, char *, int));
-void args_help PROTO((void));
-void display_args_err PROTO((char *, char **, int));
 #ifdef	_WINDOWS
 #endif
 
@@ -71,6 +69,7 @@ void display_args_err PROTO((char *, char **, int));
  * Various error and informational strings..
  */
 char args_err_missing_pinerc[] =	"missing argument for option \"-pinerc\" (use - for standard out)";
+char args_err_missing_aux[] =		"missing argument for option \"-aux\"";
 char args_err_missing_lu[] =		"missing argument for option \"-create_lu\"\nUsage: pine -create_lu <addrbook_file> <addrbook_sort_type>";
 char args_err_missing_sort[] =		"missing argument for option \"-sort\"";
 char args_err_missing_flag_arg[] =	"missing argument for flag \"%c\"";
@@ -83,6 +82,8 @@ char args_err_unknown[] =		"unknown flag \"%c\"";
 char args_err_I_error[] =		"-I argument \"%s\": %s";
 char args_err_d_error[] =		"-d argument \"%s\": %s";
 char args_err_internal[] =		"%s";
+char args_err_missing_copyprc[] =	"missing argument for option \"-copy_pinerc\"\nUsage: pine -copy_pinerc <local_pinerc> <remote_pinerc>";
+char args_err_missing_copyabook[] =	"missing argument for option \"-copy_abook\"\nUsage: pine -copy_abook <local_abook> <remote_abook>";
 
 
 char *args_pine_args[] = {
@@ -90,12 +91,14 @@ char *args_pine_args[] = {
 "",
 " Argument\tMeaning",
 " <addrs>...\tGo directly into composer sending to given address",
+"\t\tList multiple addresses with a single space between them.",
 "\t\tStandard input redirection is allowed with addresses.",
+"\t\tNote: Places addresses in the \"To\" field only.",
 " -attach <file>\tGo directly into composer with given file",
 " -attachlist <file-list>",
 " -attach_and_delete <file>",
 "\t\tGo to composer, attach file, delete when finished",
-"\t\tNote: Attach options can't be used it -f, -F, or -url",
+"\t\tNote: Attach options can't be used if -f, -F, or -url",
 "\t\tadded to Attachment list.  Attachlist must be the last",
 "\t\toption on the command line",
 " -bail\t\tExit if pinerc file doesn't already exist",
@@ -105,7 +108,8 @@ char *args_pine_args[] = {
 #endif
 " -f <folder>\tFolder - give folder name to open",
 " -c <number>\tContext - which context to apply to -f arg",
-" -F <file>\tFile - give file name to open and page through",
+" -F <file>\tFile - give file name to open and page through and",
+"\t\tforward as email.",
 " -h \t\tHelp - give this list of options",
 " -k \t\tKeys - Force use of function keys",
 " -z \t\tSuspend - allow use of ^Z suspension",
@@ -116,27 +120,45 @@ char *args_pine_args[] = {
 " -I <keystroke_list>   Initial keystrokes to be executed",
 " -n <number>\tEntry in index to begin on",
 " -o \t\tReadOnly - Open first folder read-only",
-" -conf\t\tConfiguration - Print out fresh global configuration",
+" -conf\t\tConfiguration - Print out fresh global configuration. The",
+"\t\tvalues of your global configuration affect all Pine users",
+"\t\ton your system unless they have overridden the values in their",
+"\t\tpinerc files.",
 " -pinerc <file>\tConfiguration - Put fresh pinerc configuration in <file>",
 " -p <pinerc>\tUse alternate .pinerc file",
 #if	!defined(DOS) && !defined(OS2)
 " -P <pine.conf>\tUse alternate pine.conf file",
 #else
+" -aux <aux_files_dir>\tUse this with remote pinerc",
 " -P <pine.conf>\tUse pine.conf file for default settings",
 #endif
+" -x <config>\tUse configuration exceptions in <config>.",
+"\t\tExceptions are used to override your default pinerc",
+"\t\tsettings for a particular platform, can be a local file or",
+"\t\ta remote folder.",
+" -v \t\tVersion - show version information",
 #if defined(OS2)
 " -w <rows>\tSet window size in rows on startup",
 #endif
 " -url <url>\tOpen the given URL",
-"\t\tNote: Can't be used it -f, -F, or -attach",
+"\t\tNote: Can't be used if -f, -F, or -attach",
+"\t\tStandard input redirection is not allowed with URLs.",
+"\t\tFor mailto URLs, 'body='text should be used in place of",
+"\t\tinput redirection.",
 " -create_lu <abook_file> <ab_sort_type>   create .lu from script",
+" -copy_pinerc <local_pinerc> <remote_pinerc>   copy local pinerc to remote",
+" -copy_abook <local_abook> <remote_abook>   copy local addressbook to remote",
+#ifdef notdef
 " -nr\t\tSpecial mode for UWIN",
 " -a\t\tSpecial anonymous mode for UWIN",
+#endif /* notdef */
 #if	defined(_WINDOWS)
 " -registry <cmd>\tWhere cmd is set,clear,dump",
 #endif
 " -<option>=<value>   Assign <value> to the pinerc option <option>",
 "\t\t     e.g. -signature-file=sig1",
+"\t\t     e.g. -color-style=no-color",
+"\t\t     e.g. -feature-list=enable-sigdashes",
 "\t\t     (Note: feature-list is additive)",
 NULL
 };
@@ -180,6 +202,7 @@ pine_args(pine_state, argc, argv, args)
     int   usage               = 0;
     int   do_use_fk           = 0;
     int   do_can_suspend      = 0;
+    int   do_version          = 0;
     struct variable *vars     = pine_state->vars;
 
     ac = argc;
@@ -237,6 +260,20 @@ Loop: while(--ac > 0)
 
 		  goto Loop;
 	      }
+#if	defined(DOS) || defined(OS2)
+	      else if(strcmp(*av, "aux") == 0){
+		  if(--ac){
+		      if((str = *++av) != NULL)
+			pine_state->aux_files_dir = cpystr(str);
+		  }
+		  else{
+		      display_args_err(args_err_missing_aux, NULL, 1);
+		      ++usage;
+		  }
+
+		  goto Loop;
+	      }
+#endif
 	      else if(strcmp(*av, "create_lu") == 0){
 		  if(ac > 2){
 		      ac -= 2;
@@ -250,10 +287,52 @@ Loop: while(--ac > 0)
 
 		  goto Loop;
 	      }
+	      else if(strcmp(*av, "copy_pinerc") == 0){
+		  if(args->action == aaFolder && !args->data.folder){
+		      args->action = aaPrcCopy;
+		      if(ac > 2){
+			ac -= 2;
+			args->data.copy.local  = *++av;
+			args->data.copy.remote = *++av;
+		      }
+		      else{
+			  display_args_err(args_err_missing_copyprc, NULL, 1);
+			  ++usage;
+		      }
+		  }
+		  else{
+		      display_args_err(args_err_conflict, NULL, 1);
+		      ++usage;
+		  }
+
+		  goto Loop;
+	      }
+	      else if(strcmp(*av, "copy_abook") == 0){
+		  if(args->action == aaFolder && !args->data.folder){
+		      args->action = aaAbookCopy;
+		      if(ac > 2){
+			ac -= 2;
+			args->data.copy.local  = *++av;
+			args->data.copy.remote = *++av;
+		      }
+		      else{
+			  display_args_err(args_err_missing_copyabook, NULL, 1);
+			  ++usage;
+		      }
+		  }
+		  else{
+		      display_args_err(args_err_conflict, NULL, 1);
+		      ++usage;
+		  }
+
+		  goto Loop;
+	      }
+#ifdef notdef
 	      else if(strcmp(*av, "nr") == 0){
 		  pine_state->nr_mode = 1;
 		  goto Loop;
 	      }
+#endif /* notdef */
 	      else if(strcmp(*av, "sort") == 0){
 		  if(--ac){
 		      sort = *++av;
@@ -363,21 +442,24 @@ Loop: while(--ac > 0)
 			  pine_state->update_registry = 1;
 		      }
 		      else if(!strucmp(*av, "clear")){
-			  mswin_reg(MSWR_OP_BLAST, MSWR_NULL, NULL, 0);
-			  display_args_err(
-				      "Pine related Registry values removed.",
-				      NULL, 0);
+			  if(!mswin_reg(MSWR_OP_BLAST, MSWR_NULL, NULL, 0))
+			    display_args_err(
+				       "Pine related Registry values removed.",
+				       NULL, 0);
+			  else
+			    display_args_err(
+		      "Not all Pine related Registry values could be removed",
+				       NULL, 0);
 			  exit(0);
 		      }
 		      else if(!strucmp(*av, "dump")){
-#if	0
-			  char *pRegistry[] = mswin_reg_dump();
+			  char **pRegistry = mswin_reg_dump();
 
 			  if(pRegistry){
 			      display_args_err(NULL, pRegistry, 0);
 			      free_list_array(&pRegistry);
 			  }
-#endif
+			  exit(0);
 		      }
 		      else{
 			  display_args_err("unknown registry command",
@@ -403,9 +485,11 @@ Loop: while(--ac > 0)
 		    case 'k':
 		      do_use_fk = 1;
 		      break;
+#ifdef notdef
 		    case 'a':
 		      anonymous = 1;
 		      break;
+#endif /* notdef */
 		    case 'z':
 		      do_can_suspend = 1;
 		      break;
@@ -418,10 +502,13 @@ Loop: while(--ac > 0)
 		    case 'i':
 		      pine_state->start_in_index = 1;
 		      break;
+		    case 'v':
+		      do_version = 1;
+		      break;			/* break back to inner-while */
 		      /* these take arguments */
 		    case 'f': case 'F': case 'p': case 'I':
-		    case 'c': case 'd': case 'P':	/* string args */
-		    case 'n':				/* integer args */
+		    case 'c': case 'd': case 'P': case 'x':  /* string args */
+		    case 'n':				     /* integer args */
 #ifdef	OS2
 		    case 'w':
 #endif
@@ -466,36 +553,71 @@ Loop: while(--ac > 0)
 			  break;
 			case 'p':
 			  if(str){
-			      if(is_absolute_path(str)){
-				  pine_state->pinerc = cpystr(str);
+			      char path[MAXPATH], dir[MAXPATH];
+				
+			      if(IS_REMOTE(str) || is_absolute_path(str)){
+				  strncpy(path, str, sizeof(path)-1);
+				  path[sizeof(path)-1] = '\0';
 			      }
 			      else{
-				  char path[MAXPATH], dir[MAXPATH];
+				  getcwd(dir, sizeof(path));
+				  build_path(path, dir, str, sizeof(path));
+			      }
+
+			      /*
+			       * Pinerc used to be the name of the pinerc
+			       * file. Now, since the pinerc can be remote,
+			       * we've replaced the variable pinerc with the
+			       * structure prc. Unfortunately, other parts of
+			       * pine rely on the fact that pinerc is the
+			       * name of the pinerc _file_, and use the
+			       * directory that the pinerc file is located
+			       * in for their own purposes. We keep that so
+			       * things will keep working.
+			       */
+
+			      if(!IS_REMOTE(path)){
+				  if(pine_state->pinerc)
+				    fs_give((void **)&pine_state->pinerc);
 
-				  getcwd(dir, MAXPATH);
-				  build_path(path, dir, str);
 				  pine_state->pinerc = cpystr(path);
 			      }
+
+			      /*
+			       * Last one wins. This would be the place where
+			       * we put multiple pinercs in a list if we
+			       * were to allow that.
+			       */
+			      if(pine_state->prc)
+				free_pinerc_s(&pine_state->prc);
+
+			      pine_state->prc = new_pinerc_s(path);
 			  }
-			  else
-			    pine_state->pinerc = NULL;
 
 			  break;
 			case 'P':
 			  if(str){
-			      if(is_absolute_path(str)){
-				  pine_state->pine_conf = cpystr(str);
+			      char path[MAXPATH], dir[MAXPATH];
+
+			      if(IS_REMOTE(str) || is_absolute_path(str)){
+				  strncpy(path, str, sizeof(path)-1);
+				  path[sizeof(path)-1] = '\0';
 			      }
 			      else{
-				  char path[MAXPATH], dir[MAXPATH];
-
-				  getcwd(dir, MAXPATH);
-				  build_path(path, dir, str);
-				  pine_state->pine_conf = cpystr(path);
+				  getcwd(dir, sizeof(path));
+				  build_path(path, dir, str, sizeof(path));
 			      }
+
+			      if(pine_state->pconf)
+				free_pinerc_s(&pine_state->pconf);
+
+			      pine_state->pconf = new_pinerc_s(path);
 			  }
-			  else
-			    pine_state->pine_conf = NULL;
+
+			  break;
+			case 'x':
+			  if(str)
+			    pine_state->exceptions = cpystr(str);
 
 			  break;
 			case 'c':
@@ -651,6 +773,15 @@ Loop: while(--ac > 0)
     if(usage)
       exit(-1);
 
+    if(do_version){
+	extern char datestamp[], hoststamp[];
+
+	sprintf(tmp_20k_buf, "Pine %s built %s on %s",
+		pine_version, datestamp, hoststamp);
+	display_args_err(tmp_20k_buf, NULL, 0);
+	exit(0);
+    }
+
     if(do_conf)
       dump_global_conf();
 
@@ -781,18 +912,18 @@ process_debug_str(debug_str)
 	}
 	else{
 	    debug = atoi(debug_str);
-	    if(debug > 8){
+	    if(debug > 7){
 		ps_global->debug_imap = 4;
 		ps_global->debug_timestamp = 1;
 	    }
 	    else if(debug > 6)
 	      ps_global->debug_imap = 3;
-	    else if(debug > 2)
+	    else if(debug > 4)
 	      ps_global->debug_imap = 2;
-	    else if(debug > 0)
+	    else if(debug > 2)
 	      ps_global->debug_imap = 1;
 
-	    if(debug > 7)
+	    if(debug > 8)
 	      ps_global->debug_flush = 1;
 	}
     }
