@@ -1,6 +1,6 @@
 /*
  *
- * $Id: context.h,v 4.22 1998/02/27 23:53:52 hubert Exp $
+ * $Id: context.h,v 4.25 2000/11/04 00:17:14 hubert Exp $
  *
  * Program:	Mailbox Context Management
  *
@@ -18,7 +18,7 @@
  * permission of the University of Washington.
  *
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1998 by the University of Washington.
+ * 1989-2000 by the University of Washington.
  *
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -38,8 +38,9 @@ void	    context_bboard PROTO((char *));
 void	    context_fqn_to_an PROTO((find_return, char *, char *));
 int	    context_isambig PROTO((char *));
 int	    context_isremote PROTO((CONTEXT_S *));
-char	   *context_digest PROTO((char *, char *, char *, char *, char *));
-char	   *context_apply PROTO((char *, CONTEXT_S *, char *));
+char	   *context_digest PROTO((char *, char *, char *, char *, char *,
+				  size_t));
+char	   *context_apply PROTO((char *, CONTEXT_S *, char *, size_t));
 long	    context_create PROTO((CONTEXT_S *, MAILSTREAM *, char *));
 MAILSTREAM *context_open PROTO((CONTEXT_S *, MAILSTREAM *, char *, long));
 long	    context_status PROTO((CONTEXT_S *, MAILSTREAM *, char *, long));
@@ -49,11 +50,9 @@ long	    context_append PROTO((CONTEXT_S *, MAILSTREAM *, char *, \
 				  STRING *));
 long	    context_append_full PROTO((CONTEXT_S *, MAILSTREAM *, char *, \
 				       char *, char *, STRING *));
+long	    context_append_multiple PROTO((CONTEXT_S *, MAILSTREAM *, char *, \
+					   append_t, void *));
 long	    context_copy PROTO((CONTEXT_S *, MAILSTREAM *, char *, char *));
 MAILSTREAM *context_same_stream PROTO((CONTEXT_S *, char *, MAILSTREAM *));
-#ifdef NEWBB
-void	    context_find_new_bboard PROTO((char *, MAILSTREAM *, char *, \
-					   char *));
-#endif
 
 #endif /* _CONTEXT_INCLUDED */
