@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: help.c,v 4.115 1999/09/14 20:21:50 mikes Exp $";
+static char rcsid[] = "$Id: help.c,v 4.119 2000/11/11 00:03:35 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: help.c,v 4.115 1999/09/14 20:21:50 mikes Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-1999 by the University of Washington.
+   1989-2000 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -287,7 +287,8 @@ helper_internal(text, frag, title, flags)
 				*p = '\0';
 
 			      sprintf(sargs.bar.title = tmp_title,
-				      "HELP -- %s",
+				      "HELP -- %.*s",
+				      ps_global->ttyo->screen_cols-10,
 				      strsquish(tmp_20k_buf +  500,
 					tmp_20k_buf,
 					ps_global->ttyo->screen_cols / 3));
@@ -469,7 +470,7 @@ help_processor(cmd, msgmap, sparms)
 	break;
 
       case MC_PRINTMSG :
-	sprintf(message, "%s ", STYLE_NAME(sparms));
+	sprintf(message, "%.60s ", STYLE_NAME(sparms));
 	if(open_printer(message) == 0){
 	    print_help(((HELP_SCROLL_S *)sparms->proc.data.p)->help_source);
 	    close_printer();
@@ -640,7 +641,7 @@ print_help_page_break(linenum, line, ins, local)
 	g_hprt->page++;			/* start on new page */
 	buf[0] = ctrl('L');
 	print_help_page_title(buf + 1, g_hprt);
-	strcat(buf, "\015\012\015\012\015\012");
+	strncat(buf, "\015\012\015\012\015\012", sizeof(buf)-strlen(buf));
 	ins = gf_line_test_new_ins(ins, line, buf, strlen(buf));
     }
 
@@ -714,7 +715,7 @@ help_name2section(url, url_len)
     struct _help_texts *t;
 #endif
 
-    sprintf(name, "%.*s", url_len, url);
+    sprintf(name, "%.*s", min(url_len,sizeof(name)), url);
 
 #ifdef	HELPFILE
     {
@@ -727,7 +728,7 @@ help_name2section(url, url_len)
 	 * Make sure index file is available,
 	 * and read appropriate record
 	 */
-	build_path(buf, ps_global->pine_dir, HELPINDEX);
+	build_path(buf, ps_global->pine_dir, HELPINDEX, sizeof(buf));
 	if(helpfile = fopen(buf, "rb")){
 	    for(t = 0; t < LASTHELP; t++){
 		i = fseek(helpfile, t * sizeof(struct hindx), SEEK_SET) == 0
@@ -784,7 +785,7 @@ get_help_text(index)
 	return(NULL);
 
     /* make sure index file is available, and read appropriate record */
-    build_path(buf, ps_global->pine_dir, HELPINDEX);
+    build_path(buf, ps_global->pine_dir, HELPINDEX, sizeof(buf));
     if(!(helpfile = fopen(buf, "rb"))){
 	q_status_message1(SM_ORDER,3,5,
 	    "No Help!  Index \"%s\" not found.", buf);
@@ -802,7 +803,7 @@ get_help_text(index)
     }
 
     /* make sure help file is open */
-    build_path(buf, ps_global->pine_dir, HELPFILE);
+    build_path(buf, ps_global->pine_dir, HELPFILE, sizeof(buf));
     if((helpfile = fopen(buf, "rb")) == NULL){
 	q_status_message2(SM_ORDER,3,5,"No Help!  \"%s\" : %s", buf,
 			  error_description(errno));
@@ -932,7 +933,10 @@ review_messages(title)
 		    if(last_space){
 			save = *last_space;
 			*last_space = '\0';
-			strcpy(cp, q);
+			strncpy(cp, q,
+				(width != resetwidth) ? sizeof(buf)-INDENT
+						      : sizeof(buf));
+			buf[sizeof(buf)-1] = '\0';
 			sstrcpy(&cur, buf);
 			sstrcpy(&cur, "\n");
 			*last_space = save;
@@ -941,7 +945,10 @@ review_messages(title)
 		    else{
 			save = q[width];
 			q[width] = '\0';
-			strcpy(cp, q);
+			strncpy(cp, q,
+				(width != resetwidth) ? sizeof(buf)-INDENT
+						      : sizeof(buf));
+			buf[sizeof(buf)-1] = '\0';
 			sstrcpy(&cur, buf);
 			sstrcpy(&cur, "\n");
 			q[width] = save;
@@ -965,7 +972,10 @@ review_messages(title)
 		else
 		  cp = buf;
 
-		strcpy(cp, q);
+		strncpy(cp, q,
+			(width != resetwidth) ? sizeof(buf)-INDENT
+					      : sizeof(buf));
+		buf[sizeof(buf)-1] = '\0';
 		sstrcpy(&cur, buf);
 		sstrcpy(&cur, "\n");
 	    }
@@ -1133,13 +1143,12 @@ gripe_newbody(ps, body, msgno, flags)
     gf_io_t      pc;
     static char *err = "Problem creating space for message text.";
     int          i;
-    char         tmp[MAX_ADDRESS], *p;
+    char         tmp[MAILTMPLEN], *p;
 
     if(store = so_get(PicoText, NULL, EDIT_ACCESS)){
 	*body = mail_newbody();
 
-	if((p = detoken_file(ps_global->VAR_SIGNATURE_FILE, NULL,
-			     2, 0, 1, NULL, NULL)) != NULL){
+	if((p = detoken(NULL, NULL, 2, 0, 1, NULL, NULL)) != NULL){
 	    if(*p)
 	      so_puts(store, p);
 
@@ -1215,7 +1224,7 @@ gripe_newbody(ps, body, msgno, flags)
 		/* dump last n keystrokes */
 		so_puts(store, "========== Latest keystrokes ==========\n");
 		while((i = key_playback(0)) != -1){
-		    sprintf(tmp, "\t%s\t(0x%04.4x)\n", pretty_command(i), i);
+		    sprintf(tmp, "\t%.20s\t(0x%04.4x)\n", pretty_command(i), i);
 		    so_puts(store, tmp);
 		}
 
@@ -1426,7 +1435,7 @@ void
 att_cur_drawer()
 {
     int	       i, dline, j;
-    char       buf[256];
+    char       buf[256+1];
 
     /* blat helpful message to screen */
     ClearBody();
