@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: imap.c,v 4.148 1999/11/04 01:27:56 mikes Exp $";
+static char rcsid[] = "$Id: imap.c,v 4.161 2001/01/19 17:33:06 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: imap.c,v 4.148 1999/11/04 01:27:56 mikes Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-1998 by the University of Washington.
+   1989-2001 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -152,7 +152,7 @@ mm_log(string, errflg)
     char *string;
     long  errflg;
 {
-    char        message[300];
+    char        message[sizeof(ps_global->c_client_error)];
     char       *occurance;
     int         was_capitalized;
     time_t      now;
@@ -209,7 +209,8 @@ mm_log(string, errflg)
       ps_global->mm_log_error = 1;
 
     if(errflg == PARSE || (errflg == ERROR && ps_global->noshow_error))
-      strcpy(ps_global->c_client_error, message);
+      strncpy(ps_global->c_client_error, message,
+	      sizeof(ps_global->c_client_error));
 
     if(ps_global->noshow_error
        || (ps_global->noshow_warn && errflg == WARN)
@@ -219,7 +220,8 @@ mm_log(string, errflg)
     /*---- Display the message ------*/
     q_status_message((errflg == ERROR) ? (SM_ORDER | SM_DING) : SM_ORDER,
 		     3, 5, message);
-    strcpy(ps_global->last_error, message);
+    strncpy(ps_global->last_error, message, sizeof(ps_global->last_error));
+    ps_global->last_error[sizeof(ps_global->last_error) - 1] = '\0';
 }
 
 
@@ -254,16 +256,16 @@ mm_notify(stream, string, errflag)
 	       (stream && stream->mailbox) ? stream->mailbox : "-no folder-",
 	       (stream && stream == ps_global->inbox_stream) ? "inboxstream" :
 		 (stream && stream == ps_global->mail_stream) ? "mailstream" :
-		   (stream) ? "abookstream?" : "nostream",
+		   (stream) ? "not inboxstream or mailstream" : "nostream",
 	       string));
 #endif
 
     sprintf(ps_global->last_error, "%.50s : %.*s",
 	    (stream && stream->mailbox) ? stream->mailbox : "-no folder-",
-	    min(MAX_SCREEN_COLS, 500-70),
+	    min(MAX_SCREEN_COLS, sizeof(ps_global->last_error)-70),
 	    string);
-    ps_global->last_error[ps_global->ttyo ? ps_global->ttyo->screen_cols : 499]
-									= '\0';
+    ps_global->last_error[ps_global->ttyo ? ps_global->ttyo->screen_cols
+		    : sizeof(ps_global->last_error)-1] = '\0';
 
     /*
      * Then either set special bits in the pine struct or
@@ -290,7 +292,8 @@ mm_notify(stream, string, errflag)
       q_status_message2(SM_MODAL, 3, 3, "Alert received while accessing \"%s\":  %s",
 			(stream && stream->mailbox)
 			  ? stream->mailbox : "-no folder-",
-			rfc1522_decode((unsigned char *)tmp_20k_buf, string, NULL));
+			rfc1522_decode((unsigned char *)tmp_20k_buf,
+				       SIZEOF_20KBUF, string, NULL));
     else if(!strncmp(string, "[UNSEEN ", 8)){
 	char *p;
 	long  n = 0;
@@ -336,7 +339,7 @@ mm_exists(stream, number)
     MAILSTREAM    *stream;
     unsigned long  number;
 {
-    long new_this_call;
+    long new_this_call, n;
     int	 exbits = 0;
 
 #ifdef DEBUG
@@ -360,7 +363,7 @@ mm_exists(stream, number)
 	    /*
 	     * Set local "recent" and "hidden" bits...
 	     */
-	    for(; new_this_call; --new_this_call, --number){
+	    for(n = 0; n < new_this_call; n++, number--){
 		if(msgno_exceptions(stream, number, "0", &exbits, FALSE))
 		  exbits |= MSG_EX_RECENT;
 		else
@@ -373,7 +376,9 @@ mm_exists(stream, number)
 		 * it couldn't have possibly been selected yet...
 		 */
 		if(any_lflagged(ps_global->msgmap, MN_HIDE))
-		  set_lflag(stream, ps_global->msgmap, number, MN_HIDE, 1);
+		  set_lflag(stream, ps_global->msgmap, 
+			    mn_get_total(ps_global->msgmap) - n, 
+			    MN_HIDE, 1);
 	    }
 	}
     } else if(stream == ps_global->inbox_stream) {
@@ -389,7 +394,7 @@ mm_exists(stream, number)
 	    /*
 	     * Set local "recent" and "hidden" bits...
 	     */
-	    for(; new_this_call; --new_this_call, --number){
+	    for(n = 0; n < new_this_call; n++, number--){
 		if(msgno_exceptions(stream, number, "0", &exbits, FALSE))
 		  exbits |= MSG_EX_RECENT;
 		else
@@ -402,7 +407,9 @@ mm_exists(stream, number)
 		 * it couldn't have possibly been selected yet...
 		 */
 		if(any_lflagged(ps_global->inbox_msgmap, MN_HIDE))
-		  set_lflag(stream,ps_global->inbox_msgmap,number,MN_HIDE,1);
+		  set_lflag(stream,ps_global->inbox_msgmap,
+			    mn_get_total(ps_global->inbox_msgmap) - n,
+			    MN_HIDE,1);
 	    }
 	}
     }
@@ -546,6 +553,9 @@ mm_searched(stream, number)
          user   -- Buffer to return the user name in 
          passwd -- Buffer to return the passwd in
          trial  -- The trial number or number of attempts to login
+    user is at least size NETMAXUSER
+    passwd is apparently at least MAILTMPLEN, but mrc has asked us to
+      use a max size of about 100 instead
 
  Result: username and password passed back to imap
   ----*/
@@ -556,19 +566,20 @@ mm_login(mb, user, pwd, trial)
     char   *pwd;
     long    trial;
 {
-    static char junk[] = {'?', '\0'};
     char      prompt[MAX_SCREEN_COLS], *last, *host;
     STRLIST_S hostlist[2];
     HelpType  help ;
-    int       i, j, goal, ugoal, rc, q_line, flags;
+    int       i, j, goal, ugoal, len, rc, q_line, flags;
+#define NETMAXPASSWD 100
 
-    q_line = -FOOTER_ROWS(ps_global);			/* 3 from bottom */
+    q_line = -(ps_global->ttyo ? ps_global->ttyo->footer_rows : 3);
 
     if(ps_global->anonymous) {
         /*------ Anonymous login mode --------*/
         if(trial < 1) {
             strcpy(user, "anonymous");
-            sprintf(pwd, "%s@%s", ps_global->VAR_USER_ID,
+            sprintf(pwd, "%s@%s",
+		    ps_global->VAR_USER_ID ? ps_global->VAR_USER_ID : "?",
 		    ps_global->hostname);
 	}
 	else
@@ -578,7 +589,8 @@ mm_login(mb, user, pwd, trial)
     }
 
     /* make sure errors are seen */
-    flush_status_messages(0);
+    if(ps_global->ttyo)
+      flush_status_messages(0);
 
     /*
      * set up host list for sybil servers...
@@ -598,7 +610,10 @@ mm_login(mb, user, pwd, trial)
      *  or 2) the user name we're running under.
      */
     if(trial == 0L){
-	strcpy(user, (*mb->user) ? mb->user : ps_global->VAR_USER_ID);
+	strncpy(user, (*mb->user) ? mb->user :
+		       ps_global->VAR_USER_ID ? ps_global->VAR_USER_ID : "",
+		       NETMAXUSER);
+	user[NETMAXUSER-1] = '\0';
 
 	/* try last working password associated with this host. */
 	if(imap_get_passwd(mm_login_list, pwd, user, hostlist, mb->altflag))
@@ -621,18 +636,24 @@ mm_login(mb, user, pwd, trial)
 	 */
 	if(!*mb->user
 	   && (last = imap_get_user(mm_login_list, hostlist, mb->altflag)))
-	  strcpy(user, last);
+	  strncpy(user, last, NETMAXUSER);
     }
 
+    user[NETMAXUSER-1] = '\0';
+
     ps_global->mangled_footer = 1;
     if(!*mb->user){
 	help = NO_HELP;
 
 	/* Dress up long hostnames */
-	strcpy(prompt, "HOST: ");
+	sprintf(prompt, "%sHOST: ",
+		(!ps_global->ttyo && mb->altflag) ? "+ " : "");
+	len = strlen(prompt);
 	/* leave space for "HOST", "ENTER NAME", and 15 chars for input name */
-	goal = ps_global->ttyo->screen_cols - (6 + 20
-				    + min(15, ps_global->ttyo->screen_cols/5));
+	goal = (ps_global->ttyo ? ps_global->ttyo->screen_cols : 80) -
+		(len + 20 +
+		 min(15,
+		     (ps_global->ttyo ? ps_global->ttyo->screen_cols : 80)/5));
 	last = "  ENTER LOGIN NAME: ";
 	if(goal < 9){
 	    last = " LOGIN: ";
@@ -643,8 +664,9 @@ mm_login(mb, user, pwd, trial)
 	}
 
 	if(goal){
-	    for(i = 6, j = 0; prompt[i] = mb->host[j]; i++, j++)
-	      if(i == goal && mb->host[goal+1]){
+	    for(i = len, j = 0;
+		i < sizeof(prompt) && (prompt[i] = mb->host[j]); i++, j++)
+	      if(i == goal && mb->host[goal+1] && i < sizeof(prompt)){
 		  strcpy(&prompt[i-3], "...");
 		  break;
 	      }
@@ -652,13 +674,15 @@ mm_login(mb, user, pwd, trial)
 	else
 	  i = 0;
 
-	strcpy(&prompt[i], last);
+	strncpy(&prompt[i], last, sizeof(prompt)-strlen(prompt));
+	prompt[sizeof(prompt)-1] = '\0';
 
 	while(1) {
-	    mm_login_alt_cue(mb);
+	    if(ps_global->ttyo)
+	      mm_login_alt_cue(mb);
 
 	    flags = OE_APPEND_CURRENT;
-	    rc = optionally_enter(user, q_line, 0, MAILTMPLEN - 1,
+	    rc = optionally_enter(user, q_line, 0, NETMAXUSER,
 				  prompt, NULL, help, &flags);
 	    if(rc == 3) {
 		help = help == NO_HELP ? h_oe_login : NO_HELP;
@@ -674,7 +698,10 @@ mm_login(mb, user, pwd, trial)
 	}
     }
     else
-      strcpy(user, mb->user);
+      strncpy(user, mb->user, NETMAXUSER);
+
+    user[NETMAXUSER-1] = '\0';
+    pwd[NETMAXPASSWD-1] = '\0';
 
     if(!user[0])
       return;
@@ -683,13 +710,19 @@ mm_login(mb, user, pwd, trial)
 
     /* Dress up long host/user names */
     /* leave space for "HOST", "USER" "ENTER PWD", 12 for user 6 for pwd */
+    sprintf(prompt, "%sHOST: ",
+	    (!ps_global->ttyo && mb->altflag) ? "+ " : "");
+    len = strlen(prompt);
     goal  = strlen(mb->host);
     ugoal = strlen(user);
-    if((i = ps_global->ttyo->screen_cols - (6 + 8 + 18 + 6)) < 14){
+    if((i = (ps_global->ttyo ? ps_global->ttyo->screen_cols : 80) -
+		(len + 8 + 18 + 6)) < 14){
 	goal = 0;		/* no host! */
-	if((i = ps_global->ttyo->screen_cols - (6 + 18 + 6)) <= 6){
+	if((i = (ps_global->ttyo ? ps_global->ttyo->screen_cols : 80) -
+		(6 + 18 + 6)) <= 6){
 	    ugoal = 0;		/* no user! */
-	    if((i = ps_global->ttyo->screen_cols - (18 + 6)) <= 0)
+	    if((i = (ps_global->ttyo ? ps_global->ttyo->screen_cols : 80) -
+		(18 + 6)) <= 0)
 	      i = 0;
 	}
 	else{
@@ -705,9 +738,11 @@ mm_login(mb, user, pwd, trial)
 	  ugoal--;
 
     if(goal){
-	strcpy(prompt, "HOST: ");
-	for(i = 6, j = 0; prompt[i] = mb->host[j]; i++, j++)
-	  if(i == goal && mb->host[goal+1]){
+	sprintf(prompt, "%sHOST: ",
+		(!ps_global->ttyo && mb->altflag) ? "+ " : "");
+	for(i = len, j = 0;
+	    i < sizeof(prompt) && (prompt[i] = mb->host[j]); i++, j++)
+	  if(i == goal && mb->host[goal+1] && i < sizeof(prompt)){
 	      strcpy(&prompt[i-3], "...");
 	      break;
 	  }
@@ -716,22 +751,24 @@ mm_login(mb, user, pwd, trial)
       i = 0;
 
     if(ugoal){
-	strcpy(&prompt[i], &"  USER: "[i ? 0 : 2]);
-	for(i += strlen(&prompt[i]), j = 0; prompt[i] = user[j]; i++, j++)
-	  if(j == ugoal && user[ugoal+1]){
+	strncpy(&prompt[i], &"  USER: "[i ? 0 : 2], sizeof(prompt)-i);
+	for(i += strlen(&prompt[i]), j = 0;
+	    i < sizeof(prompt) && (prompt[i] = user[j]); i++, j++)
+	  if(j == ugoal && user[ugoal+1] && i < sizeof(prompt)){
 	      strcpy(&prompt[i-3], "...");
 	      break;
 	  }
     }
 
-    strcpy(&prompt[i], &"  ENTER PASSWORD: "[i ? 0 : 8]);
+    strncpy(&prompt[i], &"  ENTER PASSWORD: "[i ? 0 : 8], sizeof(prompt)-i);
 
     *pwd = '\0';
     while(1) {
-	mm_login_alt_cue(mb);
+	if(ps_global->ttyo)
+	  mm_login_alt_cue(mb);
 
 	flags = OE_PASSWD;
-        rc = optionally_enter(pwd, q_line, 0, MAILTMPLEN - 1,
+        rc = optionally_enter(pwd, q_line, 0, NETMAXPASSWD,
 			      prompt, NULL, help, &flags);
         if(rc == 3) {
             help = help == NO_HELP ? h_oe_passwd : NO_HELP;
@@ -743,13 +780,7 @@ mm_login(mb, user, pwd, trial)
     }
 
     if(rc == 1 || !pwd[0]) {
-	if(!*mb->user){
-	    strcpy(user, junk);
-	    strcpy(pwd, junk);
-	}
-	else
-	  user[0] = pwd[0] = '\0';
-
+        user[0] = pwd[0] = '\0';
         return;
     }
 
@@ -767,7 +798,7 @@ void
 mm_login_alt_cue(mb)
     NETMBX *mb;
 {
-    if(mb->altflag){
+    if(ps_global->ttyo){
 	COLOR_PAIR  *lastc;
 
 	lastc = pico_set_colors(ps_global->VAR_TITLE_FORE_COLOR,
@@ -775,7 +806,8 @@ mm_login_alt_cue(mb)
 				PSC_REV | PSC_RET);
 
 	mark_titlebar_dirty();
-	PutLine0(0, ps_global->ttyo->screen_cols - 1, "+");
+	PutLine0(0, ps_global->ttyo->screen_cols - 1, 
+		 mb->altflag ? "+" : " ");
 
 	if(lastc){
 	    (void)pico_set_colorp(lastc, PSC_NONE);
@@ -915,7 +947,8 @@ mm_diskerror (stream, errcode, serious)
 		char full_filename[MAXPATH+1], filename[MAXPATH+1];
 
 		filename[0] = '\0';
-		build_path(full_filename, ps_global->home_dir, filename);
+		build_path(full_filename, ps_global->home_dir, filename,
+			   sizeof(full_filename));
 		file_lister("DISK ERROR", full_filename, MAXPATH+1, 
                              filename, MAXPATH+1, FALSE, FB_SAVE);
 	    }
@@ -1076,12 +1109,6 @@ mm_lsub(stream, delimiter, mailbox, attributes)
 }
 
 
-/*
- * Seconds afterwhich it's ok to prompt for connention severage...
- */
-#define	TO_BAIL_THRESHOLD	60L
-
-
 /*
  * pine_tcptimeout - C-client callback to handle tcp-related timeouts.
  */
@@ -1107,7 +1134,7 @@ pine_tcptimeout(elapsed, sincelast)
      * Prompt after a minute (since by then things are probably really bad)
      * A prompt timeout means "keep trying"...
      */
-    if(elapsed >= TO_BAIL_THRESHOLD){
+    if(elapsed >= (long)ps_global->tcp_query_timeout){
 	int clear_inverse;
 
 	ClearLine(ps_global->ttyo->screen_rows - FOOTER_ROWS(ps_global));
@@ -1726,7 +1753,7 @@ read_passfile(pinerc, l)
     if(!passfile_name(pinerc, tmp) || !(fp = fopen(tmp, "r")))
       return(0);
 
-    for(n = 0; fgets(tmp, MAILTMPLEN, fp); n++){
+    for(n = 0; fgets(tmp, sizeof(tmp), fp); n++){
 	/*** do any necessary DEcryption here ***/
 	xlate_key = n;
 	for(i = 0; tmp[i]; i++)
@@ -1775,7 +1802,7 @@ write_passfile(pinerc, l)
 
     for(n = 0; l; l = l->next, n++){
 	/*** do any necessary ENcryption here ***/
-	sprintf(tmp, "%s\t%s\t%s\t%d\n", l->passwd, l->user,
+	sprintf(tmp, "%.100s\t%.100s\t%.100s\t%d\n", l->passwd, l->user,
 		l->hosts->name, l->altflag);
 	xlate_key = n;
 	for(i = 0; tmp[i]; i++)
