@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: mailcap.c,v 1.73 1999/04/21 00:13:22 mikes Exp $";
+static char rcsid[] = "$Id: mailcap.c,v 1.76 2000/11/13 20:39:39 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: mailcap.c,v 1.73 1999/04/21 00:13:22 mikes Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-1998 by the University of Washington.
+   1989-2000 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -215,7 +215,7 @@ mc_conf_path(def_path, env_path, user_file, separator, stdpath)
 	sprintf(tmp_20k_buf + strlen(tmp_20k_buf), "%s%c%s\\%s",
 		user_file, separator, ps_global->pine_dir, user_file);
 #else	/* !DOS */
-	build_path(tmp_20k_buf, ps_global->home_dir, stdpath);
+	build_path(tmp_20k_buf, ps_global->home_dir, stdpath, SIZEOF_20KBUF);
 #endif	/* !DOS */
 	path = cpystr(tmp_20k_buf);
     }
@@ -255,7 +255,8 @@ mc_init()
 	MailcapEntry *mc = mc_new_entry();
 	STRINGLIST   *sl;
 
-	sprintf(image_viewer, "%s %%s", ps_global->VAR_IMAGE_VIEWER);
+	sprintf(image_viewer, "%.*s %%s", sizeof(image_viewer)-30,
+		ps_global->VAR_IMAGE_VIEWER);
 
 	MailcapData.raw		   = mail_newstringlist();
 	MailcapData.raw->text.data = (unsigned char *) cpystr(image_viewer);
@@ -317,7 +318,7 @@ char *file;
 
     (void)strncpy(filebuf, file, MAXPATH);
     filebuf[MAXPATH] = '\0';
-    file = fnexpand(filebuf, MAXPATH);
+    file = fnexpand(filebuf, sizeof(filebuf));
     dprint(7, (debugfile, "mailcap: processing file: %s\n", file));
     switch(is_writable_dir(file)){
       case 0: case 1: /* is a directory */
@@ -655,7 +656,9 @@ mc_get_command(type, subtype, params)
 		    strcpy(ext = tmp_ext, e2b.from.ext - 1); /* remember it */
 		    if(mt_srch_mime_type(mt_srch_by_ext, &e2b)){
 			type = e2b.to.mime.type;		/* mapped type */
-			strcpy(subtype = tmp_subtype, e2b.to.mime.subtype);
+			strncpy(subtype = tmp_subtype, e2b.to.mime.subtype,
+				sizeof(tmp_subtype)-1);
+			tmp_subtype[sizeof(tmp_subtype)-1] = '\0';
 			fs_give((void **) &e2b.to.mime.subtype);
 			params = NULL;		/* they no longer apply */
 		    }
