@@ -1,4 +1,4 @@
-# $Id: makefile.hpp,v 4.41 1998/08/27 16:49:32 hubert Exp $
+# $Id: makefile.hpp,v 4.42 2000/11/10 00:56:17 hubert Exp $
 #
 #            T H E    P I N E    M A I L   S Y S T E M
 #
@@ -58,7 +58,7 @@ PICODIR=     ../pico
 #LDAPCFLAGS=  -DENABLE_LDAP -I../ldap/include
 # Object files that need to be rebuilt if ENABLE_LDAP gets defined.
 LDAPOFILES=   addrbook.o adrbkcmd.o args.o bldaddr.o init.o \
-	      other.o pine.o takeaddr.o
+	      mailview.o other.o pine.o strings.o takeaddr.o
 
 STDLIBS=     -ltermcap
 LOCLIBS=     $(PICODIR)/libpico.a $(CCLIENTDIR)/c-client.a
@@ -110,7 +110,7 @@ $(OFILES):						$(HFILES)
 addrbook.o adrbkcmd.o adrbklib.o bldaddr.o takeaddr.o:	adrbklib.h
 context.o:						$(CCLIENTDIR)/misc.h
 send.o:							$(CCLIENTDIR)/smtp.h
-$(LDAPOFILES):						$(LDAPLIBS)
+#$(LDAPOFILES):						$(LDAPLIBS)
 
 helptext.c:	pine.hlp
 		./cmplhelp.sh  < pine.hlp > helptext.c
