@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: newmail.c,v 4.103 1999/11/02 00:12:32 mikes Exp $";
+static char rcsid[] = "$Id: newmail.c,v 4.109 2000/11/13 20:39:39 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: newmail.c,v 4.103 1999/11/02 00:12:32 mikes Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-1999 by the University of Washington.
+   1989-2000 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -131,9 +131,10 @@ new_mail(force, time_for_check_point, flags)
 			  ? BadTime : VeryBadTime, flags)
        && checknow){
 	if((flags & NM_STATUS_MSG) && F_ON(F_SHOW_DELAY_CUE, ps_global)
-	   && !ps_global->in_init_seq)
+	   && !ps_global->in_init_seq){
 	    check_cue_display(" *");	/* Show something to indicate delay */
 	    MoveCursor(ps_global->ttyo->screen_rows -FOOTER_ROWS(ps_global),0);
+	}
 #ifdef _WINDOWS
 	mswin_setcursor (MSWIN_CURSOR_BUSY);
 #endif
@@ -164,9 +165,10 @@ new_mail(force, time_for_check_point, flags)
     if(checknow && pine_state->inbox_stream 
        && stream != pine_state->inbox_stream){
 	if((flags & NM_STATUS_MSG)
-	   && F_ON(F_SHOW_DELAY_CUE, ps_global))
+	   && F_ON(F_SHOW_DELAY_CUE, ps_global)){
 	  check_cue_display(" *"); 	/* Show something to indicate delay */
 	  MoveCursor(ps_global->ttyo->screen_rows -FOOTER_ROWS(ps_global),0);
+	}
 #ifdef _WINDOWS
 	mswin_setcursor (MSWIN_CURSOR_BUSY);
 #endif
@@ -316,8 +318,8 @@ new_mail_mess(stream, folder, number, max_num)
      char       *folder;
 {
     ENVELOPE	*e;
-    char	 subject[200], from[2*MAX_SCREEN_COLS],
-		 intro[MAX_SCREEN_COLS+1];
+    char	*subject = NULL, *from = NULL, tmp[MAILTMPLEN],
+		 intro[MAX_SCREEN_COLS+1], subj_leadin[MAILTMPLEN];
     static char *carray[] = { "regarding",
 				"concerning",
 				"about",
@@ -335,37 +337,35 @@ new_mail_mess(stream, folder, number, max_num)
     e = mail_fetchstructure(stream, max_num, NULL);
 
     if(e && e->from){
-	sprintf(from, " %srom ", (number > 1L) ? "Most recent f" : "F");
-	if(e->from->personal)
-	  istrncpy(from + ((number > 1L) ? 18 : 6),
-		   (char *) rfc1522_decode((unsigned char *) tmp_20k_buf,
-					   e->from->personal, NULL),
-		   ps_global->ttyo->screen_cols);
-	else
-	  sprintf(from + ((number > 1L) ? 18 : 6), "%s%s%s", 
-		  e->from->mailbox,
-		  e->from->host ? "@" : "",
-		  e->from->host ? e->from->host : "");
+        if(e->from->personal && e->from->personal[0]){
+	    /* "75" gets us the first valid MIME3 chunk */
+	    sprintf(tmp, "%.75s", e->from->personal);
+ 	    from = cpystr((char *) rfc1522_decode((unsigned char *)tmp_20k_buf,
+ 						  SIZEOF_20KBUF, tmp, NULL));
+ 	}
+ 	else{
+ 	    sprintf(tmp, "%.40s%s%.40s", 
+ 		    e->from->mailbox,
+ 		    e->from->host ? "@" : "",
+ 		    e->from->host ? e->from->host : "");
+ 	    from = cpystr(tmp);
+ 	}
     }
-    else
-      from[0] = '\0';
 
     if(number <= 1L) {
-        if(e && e->subject){
-	    sprintf(subject, " %s ", carray[(unsigned)random()%12]);
-	    istrncpy(subject + strlen(subject),
-		     (char *) rfc1522_decode((unsigned char *) tmp_20k_buf,
-					     e->subject, NULL), 100);
+        if(e && e->subject && e->subject[0]){
+ 	    sprintf(tmp, "%.75s", e->subject);
+ 	    subject = cpystr((char *) rfc1522_decode((unsigned char *)tmp_20k_buf,
+ 						     SIZEOF_20KBUF, tmp, NULL));
 	}
-	else
-	  strcpy(subject, " with no subject");
-
-        if(!from[0])
-          subject[1] = toupper((unsigned char)subject[1]);
+	sprintf(subj_leadin, " %s ", carray[(unsigned)random()%12]);
+        if(!from)
+          subj_leadin[1] = toupper((unsigned char)subj_leadin[1]);
     }
-    else
-      subject[0] = '\0';
 
+    if(!subject)
+      subject = cpystr("");
+      
     if(!folder) {
         if(number > 1)
           sprintf(intro, "%ld new messages!", number);
@@ -377,29 +377,48 @@ new_mail_mess(stream, folder, number, max_num)
 	long fl, tot, newfl;
 
 	if(number > 1)
-	  sprintf(intro, "%ld messages saved to folder \"%s\"", number,
-		  folder);
+ 	  sprintf(intro, "%ld messages saved to folder \"%.80s\"",
+ 		  number, folder);
 	else
-	  sprintf(intro, "Mail saved to folder \"%s\"", folder);
+	  sprintf(intro, "Mail saved to folder \"%.80s\"", folder);
 	
 	if((fl=strlen(folder)) > 10 &&
 	   (tot=strlen(intro) + strlen(from) + strlen(subject)) >
 					   ps_global->ttyo->screen_cols - 2){
 	    newfl = max(10, fl-(tot-(ps_global->ttyo->screen_cols - 2)));
 	    if(number > 1)
-	      sprintf(intro, "%ld messages saved to folder \"...%s\"", number,
-		      folder+(fl-(newfl-3)));
+	      sprintf(intro, "%ld messages saved to folder \"...%.80s\"",
+		      number, folder+(fl-(newfl-3)));
 	    else
-	      sprintf(intro, "Mail saved to folder \"...%s\"",
+	      sprintf(intro, "Mail saved to folder \"...%.80s\"",
 		      folder+(fl-(newfl-3)));
 	}
     }
 
-    q_status_message3(SM_ASYNC | SM_DING, 0, 60,
-		      "%s%s%s", intro, from, subject);
+    q_status_message7(SM_ASYNC | SM_DING, 0, 60,
+ 		      "%s%s%s%s%s%s%s", intro,
+ 		      from ? ((number > 1L) ? " Most recent f" : " F") : "",
+ 		      from ? "rom " : "",
+ 		      from ? from : "",
+ 		      (number <= 1L) ? (subject[0] ? subj_leadin : "")
+				     : "",
+ 		      (number <= 1L) ? (subject[0] ? subject
+						   : from ? " w" : " W")
+				     : "",
+ 		      (number <= 1L) ? (subject[0] ? "" : "ith no subject")
+				     : "");
 
-    sprintf(tmp_20k_buf, "%s%s", intro, from);
+    sprintf(tmp_20k_buf, "%s%s%s%s", intro,
+	    from ? ((number > 1L) ? " Most recent f" : " F") : "",
+	    from ? "rom " : "",
+	    from ? from : "");
     icon_text(tmp_20k_buf);
+
+    if(from)
+      fs_give((void **) &from);
+
+    if(subject)
+      fs_give((void **) &subject);
 }
 
 
