@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: screen.c,v 4.126 1999/09/08 21:44:26 hubert Exp $";
+static char rcsid[] = "$Id: screen.c,v 4.130 2000/08/24 19:14:58 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: screen.c,v 4.126 1999/09/08 21:44:26 hubert Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-1999 by the University of Washington.
+   1989-2000 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -956,9 +956,8 @@ redraw_keymenu()
 				  && (ps_global->unseen_in_view		      \
 				      || (!(X)->seen			      \
 					  && (!IS_NEWS(as.stream)	      \
-					      || ((X)->recent		      \
-						  && F_ON(F_FAKE_NEW_IN_NEWS, \
-							  ps_global))))))     \
+					      || F_ON(F_FAKE_NEW_IN_NEWS,     \
+						      ps_global)))))          \
 				      ? MS_NEW : 0)
 
 #define	BAR_STATUS(X)	(((X) & MS_DEL) ? "DEL"   \
@@ -984,7 +983,7 @@ static struct titlebar_state {
 		 percent_column,
 		 page_column,
 		 screen_cols;
-    enum	 {Normal, ReadOnly, Closed} stream_status;
+    enum	 {Normal, OnlyRead, Closed} stream_status;
     TitleBarType style;
 } as, titlebar_stack;
 
@@ -1012,7 +1011,7 @@ status_string(stream, mc)
      } 
 
      string[0] = (!stream || !IS_NEWS(stream)
-		  || (mc->recent && F_ON(F_FAKE_NEW_IN_NEWS, ps_global)))
+		  || F_ON(F_FAKE_NEW_IN_NEWS, ps_global))
 		   ? 'N' : ' ';
 
      if(mc->seen)
@@ -1138,7 +1137,7 @@ set_titlebar(title, stream, cntxt, folder, msgmap, display_on_screen, style,
     as.stream	     = stream;
     as.stream_status = (!as.stream || (as.stream == ps_global->mail_stream
 				       && ps_global->dead_stream))
-			 ? Closed : as.stream->rdonly ? ReadOnly : Normal;
+			 ? Closed : as.stream->rdonly ? OnlyRead : Normal;
 
     if(as.folder_name)
       fs_give((void **)&as.folder_name);
@@ -1282,7 +1281,7 @@ format_titlebar(start_col)
     is_context        = strlen(as.context_name);
     sprintf(version, "PINE %s", pine_version); 
     ss_string         = as.stream_status == Closed ? "(CLOSED)" :
-                        (as.stream_status == ReadOnly
+                        (as.stream_status == OnlyRead
 			 && !IS_NEWS(as.stream))
                            ? "(READONLY)" : "";
     ss_len            = strlen(ss_string);
@@ -1517,7 +1516,7 @@ update_titlebar_status()
     struct variable *vars = ps_global->vars;
     COLOR_PAIR  *lastc;
     
-    if(!as.stream || as.current_msg < 0L || as.del_column < 0)
+    if(!as.stream || as.current_msg <= 0L || as.del_column < 0)
       return(1);
 
     mc = mail_elt(as.stream, mn_m2raw(as.msgmap, as.current_msg));
@@ -1535,13 +1534,14 @@ update_titlebar_status()
     }
     else if(!mc->seen && as.stream
 	    && (!IS_NEWS(as.stream)
-		|| (mc->recent && F_ON(F_FAKE_NEW_IN_NEWS, ps_global)))){
+		|| F_ON(F_FAKE_NEW_IN_NEWS, ps_global))){
 	if(as.msg_state & MS_NEW)	/* then seen */
 	  return(1);
     }
-    else if(!(IS_NEWS(as.stream) && F_ON(F_FAKE_NEW_IN_NEWS, ps_global))
-	    && as.msg_state == 0)		/* nothing to change... */
-      return(1);
+    else{
+	if(as.msg_state == 0)		/* nothing to change */
+	  return(1);
+    }
 
     as.msg_state = STATUS_BITS(mc);
     lastc = pico_set_colors(VAR_TITLE_FORE_COLOR, VAR_TITLE_BACK_COLOR,
@@ -1567,20 +1567,23 @@ update_titlebar_status()
   ----*/
 
 void
-update_titlebar_percent(line)
-    long line;
+update_titlebar_percent(new_line_number)
+    long new_line_number;
 {
     struct variable *vars = ps_global->vars;
     COLOR_PAIR *lastc;
 
-    if(as.percent_column < 0)
+    if(as.percent_column < 0 || new_line_number == as.current_line)
       return;
 
+    as.current_line = new_line_number;
+
     lastc = pico_set_colors(VAR_TITLE_FORE_COLOR, VAR_TITLE_BACK_COLOR,
 			    PSC_REV|PSC_RET);
 
     PutLine0(0, as.percent_column,
-	     percentage(as.current_line = line, as.total_lines, 0));
+	     percentage(as.current_line, as.total_lines, 0));
+
     if(lastc){
 	(void)pico_set_colorp(lastc, PSC_NONE);
 	free_color_pair(&lastc);
@@ -1610,14 +1613,17 @@ update_titlebar_lpercent(new_line_number)
 
     as.current_line = new_line_number;
 
+
+    lastc = pico_set_colors(VAR_TITLE_FORE_COLOR, VAR_TITLE_BACK_COLOR,
+			    PSC_REV|PSC_RET);
+
     sprintf(tmp_20k_buf, "%*ld of %*ld %s    ",
 	    digit_count(as.total_lines), as.current_line, 
 	    digit_count(as.total_lines), as.total_lines,
 	    percentage(as.current_line, as.total_lines, 0));
-    lastc = pico_set_colors(VAR_TITLE_FORE_COLOR, VAR_TITLE_BACK_COLOR,
-			    PSC_REV|PSC_RET);
 
     PutLine0(0, as.page_column, tmp_20k_buf);
+
     if(lastc){
 	(void)pico_set_colorp(lastc, PSC_NONE);
 	free_color_pair(&lastc);
