@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: signals.c,v 4.130 1999/11/09 23:56:19 hubert Exp $";
+static char rcsid[] = "$Id: signals.c,v 4.135 2000/12/29 23:08:21 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: signals.c,v 4.130 1999/11/09 23:56:19 hubert Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-1998 by the University of Washington.
+   1989-2000 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -54,6 +54,11 @@ static char rcsid[] = "$Id: signals.c,v 4.130 1999/11/09 23:56:19 hubert Exp $";
 
 #include "headers.h"
 
+/* nto defines SA_RESTART but doesn't support it yet (00/12/29 - hubert) */
+#if defined(NTO)
+#undef SA_RESTART
+#endif
+
 /*
  * call used by TERM and HUP handlers to quickly close streams
  */
@@ -70,6 +75,8 @@ static	SigType	usr2_signal SIG_PROTO((int));
 static	SigType	alarm_signal SIG_PROTO((int));
 static	SigType	intr_signal SIG_PROTO((int));
 
+static  int now_handling_alarms;
+
 
 
 /*----------------------------------------------------------------------
@@ -80,6 +87,7 @@ init_signals()
 {
     dprint(9, (debugfile, "init_signals()\n"));
     init_sighup();
+    now_handling_alarms++;
 #ifdef	_WINDOWS
     /* Only one signal works. */
     signal(SIGALRM, (void *)alarm_signal);
@@ -133,6 +141,7 @@ init_signals()
     signal(SIGALRM, alarm_signal);
 #endif
 
+
     signal(SIGPIPE, SIG_IGN);
     signal(SIGINT, SIG_IGN);
 
@@ -161,6 +170,8 @@ void
 end_signals(blockem)
     int blockem;
 {
+    now_handling_alarms = 0;
+
 #ifdef	_WINDOWS
     signal(SIGALRM, blockem ? SIG_IGN : SIG_DFL);
     signal(SIGHUP,  blockem ? SIG_IGN : SIG_DFL);
@@ -297,7 +308,7 @@ user_input_timeout_exit(to_hours)
 	    to_hours > 1 ? "hours" : "hour");
     fast_clean_up();
     completely_done_with_adrbks();
-    end_screen(msg);
+    end_screen(msg, 0);
     end_titlebar();
     end_keymenu();
     end_keyboard(F_ON(F_USE_FK,ps_global));
@@ -534,6 +545,9 @@ busy_alarm(seconds, msg, pc_func, init_msg)
 {
     int retval = 1;
 
+    if(!now_handling_alarms)
+      return 0;
+
     dprint(9,(debugfile, "busy_alarm(%d, %s, %p, %d)\n",
 	seconds, msg ? msg : "Busy", pc_func, init_msg));
 
@@ -945,11 +959,13 @@ ttyfix(come_back)
 	EndInverse();
 	end_keyboard(F_ON(F_USE_FK,ps_global));
 	end_tty_driver(ps_global);
-	end_screen(NULL);
+	end_screen(NULL, 0);
 #ifdef OS2
 	interrupt_ok();
 #endif
     }
+
+    return(0);
 }
 
 
